#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
Genbank AGP file format, see spec here
http://www.ncbi.nlm.nih.gov/projects/genome/assembly/agp
"""

import os
import re
import sys
import shutil
import logging

from copy import deepcopy
from collections import defaultdict
from itertools import groupby, izip_longest

from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO

from jcvi.formats.base import LineFile, must_open
from jcvi.formats.fasta import Fasta
from jcvi.formats.bed import Bed, BedLine
from jcvi.assembly.base import calculate_A50
from jcvi.utils.range import range_intersect
from jcvi.utils.iter import pairwise, flatten
from jcvi.apps.base import OptionParser, OptionGroup, ActionDispatcher, need_update


Valid_component_type = list("ADFGNOPUW")

Valid_gap_type = ("fragment", "clone",           # in v1.1, obsolete in v2.0
        "contig", "centromere", "short_arm",     # in both versions
        "heterochromatin", "telomere", "repeat", # in both versions
        "scaffold")                              # new in v2.0

Valid_orientation = ("+", "-", "0", "?", "na")

Valid_evidence = ("", "na", "paired-ends", "align_genus", "align_xgenus",
                  "align_trnscpt", "within_clone", "clone_contig", "map",
                  "strobe", "unspecified")

component_RGB = {"O" : "0,100,0",
                 "F" : "0,100,0",
                 "D" : "50,205,50",
                 "N" : "255,255,255"
                }

"""
phase 0 - (P)refinish; phase 1,2 - (D)raft;
phase 3 - (F)inished; 4 - (O)thers
"""
Phases = "PDDFO"


class AGPLine (object):

    def __init__(self, row, validate=True):

        atoms = row.split('\t')
        atoms[-1] = atoms[-1].strip()
        self.object = atoms[0]
        self.object_beg = int(atoms[1])
        self.object_end = int(atoms[2])
        self.object_span = self.object_end - self.object_beg + 1
        self.part_number = atoms[3]
        self.component_type = atoms[4]
        self.is_gap = (self.component_type in ('N', 'U'))

        if not self.is_gap:
            self.component_id = atoms[5]
            self.component_beg = int(atoms[6])
            self.component_end = int(atoms[7])
            self.component_span = self.component_end - self.component_beg + 1
            self.orientation = atoms[8].strip()
        else:
            self.gap_length = int(atoms[5])
            self.gap_type = atoms[6]
            self.linkage = atoms[7]
            self.linkage_evidence = []
            if len(atoms) > 8:
                linkage_evidence = atoms[8].strip()
                if linkage_evidence:
                    self.linkage_evidence = linkage_evidence.split(";")
            self.orientation = "na"
            self.component_id = "{0}.gap{1:03d}".format(self.gap_type, int(self.part_number))

        if validate:
            try:
                self.validate()
            except AssertionError, b:
                logging.error("%s\nerror when validating this line:\n%s" % \
                        (b, row))

    def __str__(self):

        fields = [self.object, self.object_beg, self.object_end,
                self.part_number, self.component_type]

        if not self.is_gap:
            fields += [self.component_id, self.component_beg,
                    self.component_end, self.orientation]
        else:
            fields += [self.gap_length, self.gap_type,
                    self.linkage, ";".join(self.linkage_evidence)]

        return "\t".join(str(x) for x in fields)

    __repr__ = __str__

    @property
    def bedline(self):
        # bed formatted line
        gid = self.component_id if not self.is_gap else self.gap_type
        return "\t".join((self.object, str(self.object_beg - 1),
                str(self.object_end), gid,
                self.component_type, self.orientation))

    @property
    def bedextra(self):
        # extra lines for bed12
        return "\t".join(str(x) for x in (self.object_beg - 1,
               self.object_end, component_RGB[self.component_type], 1,
               str(self.object_end - self.object_beg + 1) + ",", "0,"))

    @property
    def bed12line(self):
        # bed12 formatted line
        return self.bedline + "\t" + self.bedextra

    def gffline(self, gff_source="MGSC", gff_feat_type="golden_path_fragment"):
        # gff3 formatted line
        gff_feat_id = "".join(str(x) for x in (self.object, ".", \
                    "{0:03d}".format(int(self.part_number))))
        attributes = ";".join(("ID=" + gff_feat_id, \
                              "Name=" + self.component_id, \
                              "phase=" + self.component_type))
        orientation = "." if self.orientation == "na" else self.orientation

        return "\t".join(str(x) for x in (self.object, gff_source, \
               gff_feat_type, str(self.object_beg), str(self.object_end),\
               ".", orientation, ".", attributes))

    @property
    def isCloneGap(self):
        return (self.is_gap and self.gap_type != "fragment")

    def validate(self):
        assert self.component_type in Valid_component_type, \
                "component_type must be one of {0}"\
                .format("|".join(Valid_component_type))
        assert self.object_beg <= self.object_end, \
                "object_beg needs to be <= object_end"

        if not self.is_gap:
            assert self.component_beg <= self.component_end, \
                    "component_begin must be <= component_end"
            assert self.object_span == self.component_span, \
                    "object_span (%d) must be same as component_span (%d)" %\
                    (self.object_span, self.component_span)
        else:
            assert self.gap_length >= 1, \
                    "gap_length must be >= 1"
            assert self.object_span == self.gap_length, \
                    "object span (%d) must be same as gap_length (%d)" % \
                    (self.object_span, self.gap_length)
            assert all(x in Valid_evidence for x in self.linkage_evidence), \
                    "linkage_evidence must be one of {0}, you have {1}"\
                    .format("|".join(Valid_evidence), self.linkage_evidence)

            if self.linkage == "no":
                assert not self.linkage_evidence or self.linkage_evidence[0] in ("", "na"), \
                    "linkage no is incompatible with evidence {0}" \
                    .format(self.linkage_evidence)

    @classmethod
    def agpline(cls, tuple):
        return AGPLine("\t".join(str(x) for x in tuple), validate=False)

    @classmethod
    def cline(cls, object, cid, sizes, o):
        line = [object, 0, 0, 0]
        line += ['W', cid, 1, sizes[cid], o]
        return AGPLine.agpline(line)

    @classmethod
    def gline(cls, object, gap, unknown=100):
        line = [object, 0, 0, 0]
        gtype = 'N'
        if gap < unknown:
            gtype = 'U'
            gap = unknown  # Reset it to 100
        line += [gtype, gap, "scaffold", "yes", "paired-ends"]
        return AGPLine.agpline(line)


class AGP (LineFile):

    def __init__(self, filename, validate=True, sorted=True):
        super(AGP, self).__init__(filename)

        fp = open(filename)
        for row in fp:
            if row[0] == '#':
                continue
            if row.strip() == "":
                continue
            self.append(AGPLine(row, validate=validate))

        self.validate = validate
        if validate:
            if not sorted:
                self.sort(key=lambda x: (x.object, x.object_beg))
            self.validate_all()

    @property
    def order(self):
        """
        Returns a dict with component_id => (i, agpline)
        """
        d = {}
        for (i, x) in enumerate(self):
            if x.is_gap:
                continue
            xid = x.component_id
            d[xid] = (i, x)

            xid = xid.rsplit(".", 1)[0]  # Remove Genbank version
            if xid not in d:
                d[xid] = (i, x)

        return d

    def getAdjacentClone(self, i, south=True):
        """
        Returns the adjacent clone name.
        """
        rr = xrange(i + 1, len(self)) if south else xrange(i - 1, -1, -1)
        a = self[i]
        for ix in rr:
            x = self[ix]
            if x.object != a.object:
                break
            if x.is_gap:
                if x.isCloneGap:
                    return x
                else:
                    continue
            else:
                return x
        return None

    def getNorthSouthClone(self, i):
        """
        Returns the adjacent clone name from both sides.
        """
        north = self.getAdjacentClone(i, south=False)
        south = self.getAdjacentClone(i)
        return north, south

    @classmethod
    def print_header(cls, fw=sys.stdout, organism=None,
                          taxid=None, source=None, comment=None):
        # these comments are entirely optional, modeled after maize AGP
        if organism:
            print >> fw, "# ORGANISM: {0}".format(organism)
        if taxid:
            print >> fw, "# TAX_ID: {0}".format(taxid)
        if source:
            print >> fw, "# GENOME CENTER: {0}".format(source)
        if comment:
            print >> fw, "# COMMENT: {0}".format(comment)
        fields = "object object_beg object_end part_number component_type " \
                 "component_id/gap_length component_beg/gap_type " \
                 "component_end/linkage orientation/linkage_evidence"
        print >> fw, "# FIELDS: {0}".format(", ".join(fields.split()))

    def rstats(self, object, bacs, components, scaffold_sizes, length):
        from jcvi.utils.cbook import human_size

        nbacs = len(bacs)
        nscaffolds = len(scaffold_sizes)
        a50, l50, n50 = calculate_A50(scaffold_sizes)
        l50 = human_size(l50)
        length = human_size(length)

        return (object, nbacs, components, nscaffolds, n50, l50, length)

    def iter_object(self):
        for ob, lines_with_same_ob in groupby(self, key=lambda x: x.object):
            yield ob, list(lines_with_same_ob)

    def iter_paired_components(self):
        for object, lines in self.iter_object():
            lines = [x for x in lines if not x.is_gap]
            for a, b in pairwise(lines):
                qreverse = a.orientation == '-'
                yield a, b, qreverse

    def print_to_file(self, filename, index=True):
        fw = open(filename, "w")
        for a in self:
            print >> fw, a
        fw.close()
        logging.debug("AGP file written to `{0}`.".format(filename))
        if index:
             reindex([filename, "--inplace"])

    def summary_one(self, object, lines):
        bacs = set()
        components = 0
        scaffold_sizes = []
        _scaffold_key = lambda x: x.is_gap and \
                x.linkage == "no"
        length = max(x.object_end for x in lines)

        for is_gap, scaffold in groupby(lines, key=_scaffold_key):
            if is_gap:
                continue

            scaffold = list(scaffold)
            scaffold_size = 0
            for b in scaffold:
                if b.is_gap:
                    scaffold_size += b.gap_length
                else:
                    bacs.add(b.component_id)
                    components += 1
                    scaffold_size += b.component_span

            scaffold_sizes.append(scaffold_size)

        return self.rstats(object, bacs, components, scaffold_sizes, length), \
                                  (bacs, components, scaffold_sizes, length)

    def summary_all(self):

        all_bacs = set()
        all_scaffold_sizes = []
        all_components = 0
        all_length = 0
        for ob, lines in self.iter_object():
            s, bstats = self.summary_one(ob, lines)
            yield s

            bacs, components, scaffold_sizes, length = bstats
            all_components += components
            all_bacs |= bacs
            all_scaffold_sizes.extend(scaffold_sizes)
            all_length += length

        yield self.rstats("Total", all_bacs, all_components,
                          all_scaffold_sizes, all_length)

    def validate_one(self, object, lines):
        object_beg = lines[0].object_beg
        assert object_beg == 1, \
                "object %s must start at 1 (instead of %d)" % \
                (object, object_beg)

        for a, b in pairwise(lines):
            assert b.object_beg - a.object_end == 1, \
                    "lines not continuous coords between:\n%s\n%s" % \
                    (a, b)

    def validate_all(self):
        for ob, lines in self.iter_object():
            self.validate_one(ob, lines)

    def build_one(self, object, lines, fasta, fw, newagp=None):
        """
        Construct molecule using component fasta sequence
        """
        components = []

        total_bp = 0
        for line in lines:

            if line.is_gap:
                seq = 'N' * line.gap_length
                if newagp:
                    print >> newagp, line
            else:
                seq = fasta.sequence(dict(chr=line.component_id,
                        start=line.component_beg,
                        stop=line.component_end,
                        strand=line.orientation))
                # Check for dangling N's
                if newagp:
                    trimNs(seq, line, newagp)

            components.append(seq)
            total_bp += len(seq)

            if self.validate:
                assert total_bp == line.object_end, \
                        "cumulative base pairs (%d) does not match (%d)" % \
                        (total_bp, line.object_end)

        if not newagp:
            rec = SeqRecord(Seq(''.join(components)), id=object, description="")
            SeqIO.write([rec], fw, "fasta")
            if len(rec) > 1000000:
                logging.debug("Write object %s to `%s`" % (object, fw.name))

    def build_all(self, componentfasta, targetfasta, newagp=None):
        f = Fasta(componentfasta, index=False)
        fw = open(targetfasta, "w")

        for ob, lines in self.iter_object():
            self.build_one(ob, lines, f, fw, newagp=newagp)

    @property
    def graph(self):
        from jcvi.algorithms.graph import BiGraph, BiEdge

        g = BiGraph()
        for ob, lines in self.iter_object():
            components = [x for x in lines if not x.is_gap]
            gaps = [x for x in lines if x.is_gap]
            for i, (a, b) in enumerate(pairwise(components)):
                e = BiEdge(a.component_id, b.component_id,
                           a.orientation, b.orientation,
                           length=gaps[i].gap_length)
                g.add_edge(e)
            if len(components) == 1:  # Singleton object
                a = components[0]
                g.add_node(a.component_id)

        return g

    def get_line(self, cid):
        for i, a in enumerate(self):
            if not a.is_gap and a.component_id == cid:
                return i, a
        return None, None

    # Update AGP on the fly
    def delete_line(self, a, verbose=False):
        ai, ax = self.get_line(a)
        if ai is None:
            return

        if verbose:
            msg = "* Delete line:\n{0}".format(ax)
            print >> sys.stderr, msg

        del self[ai]

    def delete_lines(self, lines, verbose=False):
        deleted = set()
        for r in lines:
            if r.is_gap:
                continue
            cid = r.component_id
            self.delete_line(cid, verbose=verbose)
            deleted.add(cid)
        return deleted

    def insert_lines(self, a, lines, after=False, delete=False, verbose=False):
        if delete:
            deleted = self.delete_lines(lines, verbose=verbose)

        ai, ax = self.get_line(a)
        if after:
            ai += 1
        for i, x in enumerate(lines):
            self.insert(ai + i, x)
        if verbose:
            tag = "after" if after else "before"
            msg = "* Insert {0} line:\n".format(tag)
            msg += "\n".join([str(ax), "-" * 60]) + "\n"
            msg += "\n".join(str(x) for x in lines)
            print >> sys.stderr, msg
        return deleted

    def update_between(self, a, b, lines, delete=True, verbose=False):
        if delete:
            deleted = self.delete_lines(lines, verbose=verbose)

        ai, ax = self.get_line(a)
        bi, bx = self.get_line(b)
        # Update
        self[ai + 1: bi] = lines
        if verbose:
            msg = "* Update between:\n"
            msg += "\n".join([str(ax), str(bx), "-" * 60]) + "\n"
            msg += "\n".join(str(x) for x in lines)
            print >> sys.stderr, msg
        return deleted

    def delete_between(self, a, b, verbose=True):
        return self.update_between(a, b, [], verbose=verbose)

    def switch_between(self, a, b, verbose=True):
        ai, ax = self.get_line(a)
        bi, bx = self.get_line(b)
        self[ai] = bx
        self[bi] = ax
        if verbose:
            msg = "* Switch between:\n"
            msg += "\n".join([str(ax), str(bx)])
            print >> sys.stderr, msg


class TPFLine (object):

    def __init__(self, line):
        args = line.split()
        self.component_id = args[0]
        self.object = args[1]
        if self.is_gap:
            self.gap_type = self.component_id
        self.orientation = args[2]

    def __str__(self):
        return "\t".join((self.component_id, self.object_id, self.orientation))

    @property
    def is_gap(self):
        return self.component_id in Valid_gap_type

    @property
    def isCloneGap(self):
        return self.is_gap and self.gap_type != "fragment"


class TPF (LineFile):

    def __init__(self, filename):
        super(TPF, self).__init__(filename)
        fp = open(filename)
        for row in fp:
            if row[0] == '#':
                continue
            self.append(TPFLine(row))

    def getAdjacentClone(self, i, south=True):
        """
        Returns adjacent clone name, either the line before or after the current
        line.
        """
        rr = xrange(i + 1, len(self)) if south else xrange(i - 1, -1, -1)
        a = self[i]
        for ix in rr:
            x = self[ix]
            if x.object != a.object:
                break
            return x
        return None

    def getNorthSouthClone(self, i):
        """
        Returns adjacent clone name on both sides.
        """
        north = self.getAdjacentClone(i, south=False)
        south = self.getAdjacentClone(i)
        return north, south


class OOLine (object):

    def __init__(self, id, component_id, component_size, strand):
        self.id = id
        self.component_id = component_id
        self.component_size = component_size
        self.strand = strand


class OO (LineFile):

    def __init__(self, filename=None, ctgsizes=None):
        super(OO, self).__init__(filename)

        if filename is None:
            return

        from jcvi.formats.base import read_block

        fp = open(filename)
        prefix = "contig_"
        self.contigs = set()
        for header, block in read_block(fp, ">"):
            header = header[1:]  # Trim the '>'
            header = header.split()[0]
            for b in block:
                ctg, orientation = b.split()
                if ctg.startswith(prefix):
                    ctg = ctg[len(prefix):]

                assert orientation in ("BE", "EB")

                strand = "+" if orientation == "BE" else "-"
                ctgsize = ctgsizes[ctg]
                self.add(header, ctg, ctgsize, strand)
                self.contigs.add(ctg)

    def add(self, scaffold, ctg, ctgsize, strand="0"):
        self.append(OOLine(scaffold, ctg, ctgsize, strand))

    def sub_beds(self):
        for scaffold, beds in groupby(self, key=lambda x: x.id):
            yield scaffold, list(beds)

    def write_AGP(self, fw=sys.stdout, gapsize=100, phases={},
                        gaptype="scaffold", evidence=""):

        linkage = "yes"

        for object, beds in self.sub_beds():
            object_beg = 1
            part_number = 0
            for b in beds:
                component_id = b.component_id
                size = b.component_size
                if part_number > 0:  # Print gap except for the first one
                    object_end = object_beg + gapsize - 1
                    part_number += 1
                    print >> fw, "\t".join(str(x) for x in \
                            (object, object_beg, object_end, part_number,
                             'N', gapsize, gaptype, linkage, evidence))

                    object_beg += gapsize

                object_end = object_beg + size - 1
                part_number += 1
                strand = '?' if b.strand == '0' else b.strand
                print >> fw, "\t".join(str(x) for x in \
                        (object, object_beg, object_end, part_number,
                         phases.get(component_id, 'W'), component_id,
                         1, size, strand))

                object_beg += size


def order_to_agp(object, ctgorder, sizes, fwagp, gapsize=100, gaptype="scaffold"):

    o = OO()  # Without a filename
    for scaffold_number, (ctg, strand) in enumerate(ctgorder):
        size = sizes[ctg]
        o.add(object, ctg, size, strand)

    o.write_AGP(fwagp, gapsize=gapsize, gaptype=gaptype, phases={})


def trimNs(seq, line, newagp):
    """
    Test if the sequences contain dangling N's on both sides. This component
    needs to be adjusted to the 'actual' sequence range.
    """
    start, end = line.component_beg, line.component_end
    size = end - start + 1
    leftNs, rightNs = 0, 0
    lid, lo = line.component_id, line.orientation
    for s in seq:
        if s in 'nN':
            leftNs += 1
        else:
            break
    for s in seq[::-1]:
        if s in 'nN':
            rightNs += 1
        else:
            break

    if lo == '-':
        trimstart = start + rightNs
        trimend = end - leftNs
    else:
        trimstart = start + leftNs
        trimend = end - rightNs

    trimrange = (trimstart, trimend)
    oldrange = (start, end)

    if trimrange != oldrange:
        logging.debug("{0} trimmed of N's: {1} => {2}".\
                        format(lid, oldrange, trimrange))

        if leftNs:
            print >> newagp, "\t".join(str(x) for x in (line.object, 0, 0, 0,
                    'N', leftNs, "fragment", "yes", ""))
        if trimend > trimstart:
            print >> newagp, "\t".join(str(x) for x in (line.object, 0, 0, 0,
                    line.component_type, lid, trimstart, trimend, lo))
        if rightNs and rightNs != size:
            print >> newagp, "\t".join(str(x) for x in (line.object, 0, 0, 0,
                    'N', rightNs, "fragment", "yes", ""))
    else:
        print >> newagp, line


def main():

    actions = (
        ('summary', 'print out a table of scaffold statistics'),
        ('stats', 'print out a report for length of gaps and components'),
        ('phase', 'given genbank file, get the phase for the HTG BAC record'),
        ('bed', 'print out the tiling paths in bed/gff3 format'),
        ('frombed', 'generate AGP file based on bed file'),
        ('extendbed', 'extend the components to fill the component range and output bed/gff3 format file'),
        ('gaps', 'print out the distribution of gap sizes'),
        ('tpf', 'print out a list of accessions, aka Tiling Path File'),
        ('cut', 'cut at the boundaries of given ranges'),
        ('mask', 'mask given ranges in components to gaps'),
        ('liftover', 'given ranges in components, get chromosome ranges'),
        ('swap', 'swap objects and components'),
        ('format', 'reformat AGP file'),
        ('reindex', 'assume accurate component order, reindex coordinates'),
        ('tidy', 'run trim=>reindex=>merge sequentially'),
        ('build', 'given agp file and component fasta file, build ' +\
                 'pseudomolecule fasta'),
        ('validate', 'given agp file, component and pseudomolecule fasta, ' +\
                     'validate if the build is correct'),
        ('infer', 'infer where the components are in the genome'),
            )

    p = ActionDispatcher(actions)
    p.dispatch(globals())


def map_one_scaffold_1way(scaffold_name, scaffold, genome, orientation='+'):
    if orientation == '-':
        scaffold = scaffold.reverse_complement()

    scaffold = str(scaffold)
    for obj_name, obj in genome.iteritems():
        obj_idx = obj.find(scaffold)
        if obj_idx == -1:
            continue
        else:
            return obj_name, obj_idx, orientation
    return -1, -1, orientation  # unmapped scaffolds


def map_one_scaffold(opts):
    scaffold_name, scaffold, genome = opts
    scaffold = scaffold.seq
    obj_name, obj_idx, objo = map_one_scaffold_1way(scaffold_name, scaffold, genome)
    if obj_name == -1:
        obj_name, obj_idx, objo = map_one_scaffold_1way(scaffold_name, scaffold, genome,
                                                   orientation='-')
    if obj_name == -1:
        return ""

    obj_end = obj_idx + len(scaffold)
    return "\t".join(str(x) for x in (obj_name, obj_idx, obj_end,
                                      scaffold_name, 1000, objo))


def check_seen(r, seen):
    from jcvi.utils.range import range_overlap

    for s in seen:
        if range_overlap(r, s):
            return True
    return False


def infer(args):
    """
    %prog infer scaffolds.fasta genome.fasta

    Infer where the components are in the genome. This function is rarely used,
    but can be useful when distributor does not ship an AGP file.
    """
    from jcvi.apps.grid import WriteJobs
    from jcvi.formats.bed import sort

    p = OptionParser(infer.__doc__)
    p.set_cpus()
    opts, args = p.parse_args(args)

    if len(args) != 2:
        sys.exit(not p.print_help())

    scaffoldsf, genomef = args
    inferbed = "infer-components.bed"
    if need_update((scaffoldsf, genomef), inferbed):
        scaffolds = Fasta(scaffoldsf, lazy=True)
        genome = Fasta(genomef)
        genome = genome.tostring()
        args = [(scaffold_name, scaffold, genome) \
                for scaffold_name, scaffold in scaffolds.iteritems_ordered()]

        pool = WriteJobs(map_one_scaffold, args, inferbed, cpus=opts.cpus)
        pool.run()

    sort([inferbed, "-i"])
    bed = Bed(inferbed)
    inferagpbed = "infer.bed"
    fw = open(inferagpbed, "w")
    seen = []
    for b in bed:
        r = (b.seqid, b.start, b.end)
        if check_seen(r, seen):
            continue
        print >> fw, "\t".join(str(x) for x in \
                    (b.accn, 0, b.span, b.seqid, b.score, b.strand))
        seen.append(r)
    fw.close()

    frombed([inferagpbed])


def format(args):
    """
    %prog format oldagpfile newagpfile

    Reformat AGP file. --switch will replace the ids in the AGP file.
    """
    from jcvi.formats.base import DictFile

    p = OptionParser(format.__doc__)
    p.add_option("--switchcomponent",
                 help="Switch component id based on")
    opts, args = p.parse_args(args)

    if len(args) != 2:
        sys.exit(not p.print_help())

    oldagpfile, newagpfile = args
    switchcomponent = opts.switchcomponent
    if switchcomponent:
        switchcomponent = DictFile(switchcomponent)

    agp = AGP(oldagpfile)
    fw = open(newagpfile, "w")
    nconverts = 0
    for i, a in enumerate(agp):
        if not a.is_gap and a.component_id in switchcomponent:
            oldid = a.component_id
            newid = switchcomponent[a.component_id]
            a.component_id = newid
            logging.debug("Covert {0} to {1} on line {2}".\
                        format(oldid, newid, i+1))
            nconverts += 1
        print >> fw, a

    logging.debug("Total converted records: {0}".format(nconverts))


def frombed(args):
    """
    %prog frombed bedfile

    Generate AGP file based on bed file. The bed file must have at least 6
    columns. With the 4-th column indicating the new object.
    """
    p = OptionParser(frombed.__doc__)
    p.add_option("--gapsize", default=100, type="int",
                 help="Insert gaps of size [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(not p.print_help())

    bedfile, = args
    gapsize = opts.gapsize
    agpfile = bedfile.replace(".bed", ".agp")
    fw = open(agpfile, "w")

    bed = Bed(bedfile, sorted=False)
    for object, beds in groupby(bed, key=lambda x: x.accn):
        beds = list(beds)
        for i, b in enumerate(beds):
            if gapsize and i != 0:
                print >> fw, "\t".join(str(x) for x in \
                    (object, 0, 0, 0, "U", \
                     gapsize, "contig", "no", "na"))

            bstrand = '-' if b.strand == '-' else '+'
            print >> fw, "\t".join(str(x) for x in \
                    (object, 0, 0, 0, "W", \
                     b.seqid, b.start, b.end, bstrand))

    fw.close()

    # Reindex
    return reindex([agpfile, "--inplace"])


def swap(args):
    """
    %prog swap agpfile

    Swap objects and components. Will add gap lines. This is often used in
    conjuction with formats.chain.fromagp() to convert between different
    coordinate systems.
    """
    from jcvi.utils.range import range_interleave

    p = OptionParser(swap.__doc__)
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(not p.print_help())

    agpfile, = args

    agp = AGP(agpfile)
    agp.sort(key=lambda x: (x.component_id, x.component_beg))

    newagpfile = agpfile.rsplit(".", 1)[0] + ".swapped.agp"
    fw = open(newagpfile, "w")
    for cid, aa in groupby(agp, key=(lambda x: x.component_id)):
        aa = list(aa)
        aranges = [(x.component_id, x.component_beg, x.component_end) \
                    for x in aa]
        gaps = range_interleave(aranges)
        for a, g in izip_longest(aa, gaps):
            a.object, a.component_id = a.component_id, a.object
            a.component_beg = a.object_beg
            a.component_end = a.object_end
            print >> fw, a
            if not g:
                continue

            aline = [cid, 0, 0, 0]
            gseq, ga, gb = g
            cspan = gb - ga + 1
            aline += ["N", cspan, "fragment", "yes"]
            print >> fw, "\t".join(str(x) for x in aline)

    fw.close()
    # Reindex
    idxagpfile = reindex([newagpfile, "--inplace"])

    return newagpfile


def stats(args):
    """
    %prog stats agpfile

    Print out a report for length of gaps and components.
    """
    from jcvi.utils.table import tabulate

    p = OptionParser(stats.__doc__)
    p.add_option("--warn", default=False, action="store_true",
                 help="Warnings on small component spans [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(p.print_help())

    agpfile, = args

    agp = AGP(agpfile)
    gap_lengths = []
    component_lengths = []
    for a in agp:
        span = a.object_span
        if a.is_gap:
            label = a.gap_type
            gap_lengths.append((span, label))
        else:
            label = "{0}:{1}-{2}".format(a.component_id, a.component_beg, \
                   a.component_end)
            component_lengths.append((span, label))
            if opts.warn and span < 50:
                logging.error("component span too small ({0}):\n{1}".\
                    format(span, a))

    table = dict()
    for label, lengths in zip(("Gaps", "Components"),
            (gap_lengths, component_lengths)):

        if not lengths:
            table[(label, "Min")] = table[(label, "Max")] \
                                  = table[(label, "Sum")] = "n.a."
            continue

        table[(label, "Min")] = "{0} ({1})".format(*min(lengths))
        table[(label, "Max")] = "{0} ({1})".format(*max(lengths))
        table[(label, "Sum")] = sum(x[0] for x in lengths)

    print >> sys.stderr, tabulate(table)


def cut(args):
    """
    %prog cut agpfile bedfile

    Cut at the boundaries of the ranges in the bedfile.
    """
    p = OptionParser(cut.__doc__)
    opts, args = p.parse_args(args)

    if len(args) != 2:
        sys.exit(not p.print_help())

    agpfile, bedfile = args
    agp = AGP(agpfile)
    bed = Bed(bedfile)
    simple_agp = agp.order
    newagpfile = agpfile.replace(".agp", ".cut.agp")
    fw = open(newagpfile, "w")

    agp_fixes = defaultdict(list)
    for component, intervals in bed.sub_beds():
        i, a = simple_agp[component]
        object = a.object
        component_span = a.component_span
        orientation = a.orientation

        assert a.component_beg, a.component_end
        arange = a.component_beg, a.component_end

        cuts = set()
        for i in intervals:
            start, end = i.start, i.end
            end -= 1

            assert start <= end
            cuts.add(start)
            cuts.add(end)

        cuts.add(0)
        cuts.add(component_span)
        cuts = list(sorted(cuts))

        sum_of_spans = 0
        for i, (a, b) in enumerate(pairwise(cuts)):
            oid = object + "_{0}".format(i)
            aline = [oid, 0, 0, 0]
            cspan = b - a
            aline += ['D', component, a + 1, b, orientation]
            sum_of_spans += cspan

            aline = "\t".join(str(x) for x in aline)
            agp_fixes[component].append(aline)

        assert component_span == sum_of_spans

    # Finally write the masked agp
    for a in agp:
        if not a.is_gap and a.component_id in agp_fixes:
            print >> fw, "\n".join(agp_fixes[a.component_id])
        else:
            print >> fw, a

    fw.close()
    # Reindex
    reindex([newagpfile, "--inplace"])

    return newagpfile


def mask(args):
    """
    %prog mask agpfile bedfile

    Mask given ranges in components to gaps.
    """
    p = OptionParser(mask.__doc__)
    p.add_option("--splitobject", default=False, action="store_true",
                 help="Create new names for object [default: %default]")
    p.add_option("--splitcomponent", default=False, action="store_true",
                 help="Create new names for component [default: %default]")
    p.add_option("--gaptype", default="scaffold",
                 help="Masked region has gap type of [default: %default]")
    p.add_option("--retain", default=False, action="store_true",
                 help="Retain old names for non-split objects [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) != 2:
        sys.exit(p.print_help())

    agpfile, bedfile = args
    gaptype = opts.gaptype
    splitobject = opts.splitobject
    splitcomponent = opts.splitcomponent

    assert not (splitobject and splitcomponent), \
                "Options --splitobject and --splitcomponent conflict"

    agp = AGP(agpfile)
    bed = Bed(bedfile)
    simple_agp = agp.order
    # agp lines to replace original ones, keyed by the component
    agp_fixes = defaultdict(list)

    newagpfile = agpfile.replace(".agp", ".masked.agp")
    fw = open(newagpfile, "w")

    if splitcomponent:
        componentindex = defaultdict(int)

    for component, intervals in bed.sub_beds():
        i, a = simple_agp[component]
        object = a.object
        component_span = a.component_span
        orientation = a.orientation

        assert a.component_beg, a.component_end
        arange = a.component_beg, a.component_end

        # Make sure `ivs` contain DISJOINT ranges, and located within `arange`
        ivs = []
        for i in intervals:
            iv = range_intersect(arange, (i.start, i.end))
            if iv is not None:
                ivs.append(iv)

        # Sort the ends of `ivs` as well as the arange
        arange = a.component_beg - 1, a.component_end + 1
        endpoints = sorted(flatten(ivs + [arange]))
        # reverse if component on negative strand
        if orientation == '-':
            endpoints.reverse()

        sum_of_spans = 0
        # assign complements as sequence components
        for i, (a, b) in enumerate(pairwise(endpoints)):
            if orientation == '-':
                a, b = b, a
            if orientation not in ('+', '-'):
                orientation = '+'

            oid = object + "_{0}".format(i / 2) if splitobject else object
            aline = [oid, 0, 0, 0]
            if i % 2 == 0:
                cspan = b - a - 1
                if splitcomponent:
                    cid = component + "_{0}".format(componentindex[component])
                    componentindex[component] += 1
                    aline += ['W', cid, 1, cspan, orientation]
                else:
                    aline += ['W', component, a + 1, b - 1, orientation]
                is_gap = False
            else:
                cspan = b - a + 1
                aline += ["N", cspan, gaptype, "yes", "paired-ends"]
                is_gap = True
            if cspan <= 0:
                continue

            sum_of_spans += cspan
            aline = "\t".join(str(x) for x in aline)
            if not (splitobject and is_gap):
                agp_fixes[component].append(aline)

    retain = opts.retain
    # Finally write the masked agp
    for a in agp:
        if a.is_gap:
            print >> fw, a
        elif a.component_id in agp_fixes:
            print >> fw, "\n".join(agp_fixes[a.component_id])
        else:
            if not retain:
                if splitobject:
                    a.object += "_0"
                elif splitcomponent:
                    a.component_id += "_0"
            print >> fw, a

    fw.close()

    # Reindex
    idxagpfile = reindex([newagpfile, "--inplace"])

    return newagpfile


def liftover(args):
    """
    %prog liftover agpfile bedfile

    Given coordinates in components, convert to the coordinates in chromosomes.
    """
    p = OptionParser(liftover.__doc__)
    p.add_option("--prefix", default=False, action="store_true",
                 help="Prepend prefix to accn names [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) != 2:
        sys.exit(p.print_help())

    agpfile, bedfile = args
    agp = AGP(agpfile).order
    bed = Bed(bedfile)
    newbed = Bed()
    for b in bed:
        component = b.seqid
        if component not in agp:
            newbed.append(b)
            continue

        i, a = agp[component]

        assert a.component_beg < a.component_end
        arange = a.component_beg, a.component_end
        assert b.start < b.end
        brange = b.start, b.end

        st = range_intersect(arange, brange)
        if not st:
            continue
        start, end = st
        assert start <= end

        if a.orientation == '-':
            d = a.object_end + a.component_beg
            s, t = d - end, d - start
        else:
            d = a.object_beg - a.component_beg
            s, t = d + start, d + end

        name = b.accn.replace(" ", "_")
        if opts.prefix:
            name = component + "_" + name
        bline = "\t".join(str(x) for x in (a.object, s - 1, t, name))
        newbed.append(BedLine(bline))

    newbed.print_to_file(sorted=True)


def reindex(args):
    """
    %prog agpfile

    assume the component line order is correct, modify coordinates, this is
    necessary mostly due to manual edits (insert/delete) that disrupts
    the target coordinates.
    """
    p = OptionParser(reindex.__doc__)
    p.add_option("--nogaps", default=False, action="store_true",
                 help="Remove all gap lines [default: %default]")
    p.add_option("--inplace", default=False, action="store_true",
                 help="Replace input file [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(p.print_help())

    agpfile, = args
    inplace = opts.inplace
    agp = AGP(agpfile, validate=False)
    pf = agpfile.rsplit(".", 1)[0]
    newagpfile = pf + ".reindexed.agp"

    fw = open(newagpfile, "w")
    for chr, chr_agp in groupby(agp, lambda x: x.object):
        chr_agp = list(chr_agp)
        object_beg = 1
        for i, b in enumerate(chr_agp):
            b.object_beg = object_beg
            b.part_number = i + 1
            if opts.nogaps and b.is_gap:
                continue

            if b.is_gap:
                b.object_end = object_beg + b.gap_length - 1
            else:
                b.object_end = object_beg + b.component_span - 1

            object_beg = b.object_end + 1

            print >> fw, str(b)

    # Last step: validate the new agpfile
    fw.close()
    agp = AGP(newagpfile, validate=True)

    if inplace:
        shutil.move(newagpfile, agpfile)
        logging.debug("Rename file `{0}` to `{1}`".format(newagpfile, agpfile))
        newagpfile = agpfile

    return newagpfile


def summary(args):
    """
    %prog summary agpfile

    print a table of scaffold statistics, number of BACs, no of scaffolds,
    scaffold N50, scaffold L50, actual sequence, PSMOL NNNs, PSMOL-length, % of
    PSMOL sequenced.
    """
    from jcvi.utils.table import write_csv

    p = OptionParser(summary.__doc__)
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(p.print_help())

    agpfile, = args
    header = "Chromosome #_Distinct #_Components #_Scaffolds " \
             "Scaff_N50 Scaff_L50 Length".split()

    agp = AGP(agpfile)
    data = list(agp.summary_all())
    write_csv(header, data, sep=" ")


chr_pat = re.compile("chromosome (\d)", re.I)
clone_pat = re.compile("clone ([^, ]*\d)[ ,]", re.I)


def get_clone(rec):
    """
    >>> get_clone("Medicago truncatula chromosome 2 clone mth2-48e18")
    ('2', 'mth2-48e18')
    """
    s = rec.description
    chr = re.search(chr_pat, s)
    clone = re.search(clone_pat, s)
    chr = chr.group(1) if chr else ""
    clone = clone.group(1) if clone else ""

    return chr, clone


def get_phase(rec):
    keywords = rec.annotations["keywords"]
    description = rec.description.upper()

    if "HTGS_PHASE1" in keywords:
        phase = 1
    elif "HTGS_PHASE2" in keywords:
        phase = 2
    elif len(keywords) == 1 and "HTG" in keywords:
        phase = 3
    elif "PLN" in keywords:  # EMBL BACs
        if "DRAFT" in description:
            if "UNORDERED" in description:
                phase = 1
            else:
                phase = 2
        else:
            assert "COMPLETE" in description, description
            phase = 3
    else:
        #logging.error("{0}: {1}".format(rec.name, description))
        phase = 3

    return phase, keywords


def phase(args):
    """
    %prog phase genbankfiles

    Input has to be gb file. Search the `KEYWORDS` section to look for PHASE.
    Also look for "chromosome" and "clone" in the definition line.
    """
    p = OptionParser(phase.__doc__)
    p.set_outfile()

    opts, args = p.parse_args(args)

    if len(args) < 1:
        sys.exit(not p.print_help())

    fw = must_open(opts.outfile, "w")
    for gbfile in args:
        for rec in SeqIO.parse(gbfile, "gb"):
            bac_phase, keywords = get_phase(rec)
            chr, clone = get_clone(rec)
            keyword_field = ";".join(keywords)
            print >> fw, "\t".join((rec.id, str(bac_phase), keyword_field,
                    chr, clone))


def tpf(args):
    """
    %prog tpf agpfile

    Print out a list of ids, one per line. Also known as the Tiling Path.

    AC225490.9  chr6

    Can optionally output scaffold gaps.
    """
    p = OptionParser(tpf.__doc__)
    p.add_option("--noversion", default=False, action="store_true",
            help="Remove trailing accession versions [default: %default]")
    p.add_option("--gaps", default=False, action="store_true",
            help="Include gaps in the output [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(not p.print_help())

    agpfile, = args
    agp = AGP(agpfile)
    for a in agp:
        object = a.object
        if a.is_gap:
            if opts.gaps and a.isCloneGap:
                print "\t".join((a.gap_type, object, "na"))
            continue

        component_id = a.component_id
        orientation = a.orientation

        if opts.noversion:
            component_id = component_id.rsplit(".", 1)[0]

        print "\t".join((component_id, object, orientation))



def bed(args):
    """
    %prog bed agpfile

    print out the tiling paths in bed/gff3 format
    """
    from jcvi.formats.obo import load_GODag, validate_term

    p = OptionParser(bed.__doc__)
    p.add_option("--gaps", default=False, action="store_true",
            help="Only print bed lines for gaps [default: %default]")
    p.add_option("--nogaps", default=False, action="store_true",
            help="Do not print bed lines for gaps [default: %default]")
    p.add_option("--bed12", default=False, action="store_true",
            help="Produce bed12 formatted output [default: %default]")
    p.add_option("--component", default=False, action="store_true",
            help="Generate bed file for components [default: %default]")
    p.set_outfile()
    g1 = OptionGroup(p, "GFF specific parameters",
            "Note: If not specified, output will be in `bed` format")
    g1.add_option("--gff", default=False, action="store_true",
            help="Produce gff3 formatted output. By default, ignores " +\
                 "AGP gap lines. [default: %default]")
    g1.add_option("--source", default="MGSC",
            help="Specify a gff3 source [default: `%default`]")
    g1.add_option("--feature", default="golden_path_fragment",
            help="Specify a gff3 feature type [default: `%default`]")
    p.add_option_group(g1)
    p.set_SO_opts()

    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(not p.print_help())

    if opts.component:
        opts.nogaps = True

    # If output format is gff3 and 'verifySO' option is invoked, validate the SO term
    if opts.gff and opts.verifySO:
        validate_term(opts.feature, method=opts.verifySO)

    agpfile, = args
    agp = AGP(agpfile)
    fw = must_open(opts.outfile, "w")
    if opts.gff:
        print >> fw, "##gff-version 3"

    for a in agp:
        if opts.nogaps and a.is_gap:
            continue
        if opts.gaps and not a.is_gap:
            continue
        if opts.bed12:
            print >> fw, a.bed12line
        elif opts.gff:
            print >> fw, a.gffline(gff_source=opts.source, gff_feat_type=opts.feature)
        elif opts.component:
            name = "{0}:{1}-{2}".\
                        format(a.component_id, a.component_beg, a.component_end)
            print >> fw, "\t".join(str(x) for x in (a.component_id, a.component_beg - 1,
                                a.component_end, name,
                                a.component_type, a.orientation))
        else:
            print >> fw, a.bedline
    fw.close()

    return fw.name


def extendbed(args):
    """
    %prog extend agpfile componentfasta

    Extend the components to fill the component range. For example, a bed/gff3 file
    that was converted from the agp will contain only the BAC sequence intervals
    that are 'represented' - sometimes leaving the 5` and 3` out (those that
    overlap with adjacent sequences. This script fill up those ranges,
    potentially to make graphics for tiling path.
    """
    from jcvi.formats.sizes import Sizes

    p = OptionParser(extendbed.__doc__)
    p.add_option("--nogaps", default=False, action="store_true",
            help="Do not print bed lines for gaps [default: %default]")
    p.add_option("--bed12", default=False, action="store_true",
            help="Produce bed12 formatted output [default: %default]")
    p.add_option("--gff", default=False, action="store_true",
            help="Produce gff3 formatted output. By default, ignores " +\
                 " AGP gap lines. [default: %default]")
    p.set_outfile()

    opts, args = p.parse_args(args)

    if len(args) != 2:
        sys.exit(not p.print_help())

    # If output format is GFF3, ignore AGP gap lines.
    if opts.gff:
        opts.nogaps = True

    agpfile, fastafile = args
    agp = AGP(agpfile)
    fw = must_open(opts.outfile, "w")
    if opts.gff:
        print >> fw, "##gff-version 3"

    ranges = defaultdict(list)
    thickCoords = []  # These are the coordinates before modify ranges
    # Make the first pass to record all the component ranges
    for a in agp:
        thickCoords.append((a.object_beg, a.object_end))
        if a.is_gap:
            continue
        ranges[a.component_id].append(a)

    # Modify the ranges
    sizes = Sizes(fastafile).mapping
    for accn, rr in ranges.items():
        alen = sizes[accn]

        a = rr[0]
        if a.orientation == "+":
            hang = a.component_beg - 1
        else:
            hang = alen - a.component_end
        a.object_beg -= hang

        a = rr[-1]
        if a.orientation == "+":
            hang = alen - a.component_end
        else:
            hang = a.component_beg - 1
        a.object_end += hang

    for a, (ts, te) in zip(agp, thickCoords):
        if opts.nogaps and a.is_gap:
            continue
        if opts.bed12:
            line = a.bedline
            a.object_beg, a.object_end = ts, te
            line += "\t" + a.bedextra
            print >> fw, line
        elif opts.gff:
            print >> fw, a.gffline()
        else:
            print >> fw, a.bedline


def gaps(args):
    """
    %prog gaps agpfile

    Print out the distribution of gapsizes. Option --merge allows merging of
    adjacent gaps which is used by tidy().
    """
    from jcvi.graphics.histogram import loghistogram

    p = OptionParser(gaps.__doc__)
    p.add_option("--merge", dest="merge", default=False, action="store_true",
            help="Merge adjacent gaps (to conform to AGP specification)")
    p.add_option("--header", default=False, action="store_true",
            help="Produce an AGP header [default: %default]")

    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(not p.print_help())

    merge = opts.merge
    agpfile, = args

    if merge:
        merged_agpfile = agpfile.replace(".agp", ".merged.agp")
        fw = open(merged_agpfile, "w")

    agp = AGP(agpfile)
    sizes = []
    data = []  # store merged AGPLine's
    priorities = ("centromere", "telomere", "scaffold", "contig", \
            "clone", "fragment")

    for is_gap, alines in groupby(agp, key=lambda x: (x.object, x.is_gap)):
        alines = list(alines)
        is_gap = is_gap[1]
        if is_gap:
            gap_size = sum(x.gap_length for x in alines)
            gap_types = set(x.gap_type for x in alines)
            for gtype in ("centromere", "telomere"):
                if gtype in gap_types:
                    gap_size = gtype

            sizes.append(gap_size)
            b = deepcopy(alines[0])
            b.object_beg = min(x.object_beg for x in alines)
            b.object_end = max(x.object_end for x in alines)
            b.gap_length = sum(x.gap_length for x in alines)

            assert b.gap_length == b.object_end - b.object_beg + 1
            b.component_type = 'U' if b.gap_length == 100 else 'N'

            gtypes = [x.gap_type for x in alines]
            for gtype in priorities:
                if gtype in gtypes:
                    b.gap_type = gtype
                    break

            linkages = [x.linkage for x in alines]
            for linkage in ("no", "yes"):
                if linkage in linkages:
                    b.linkage = linkage
                    break

            alines = [b]

        data.extend(alines)

    loghistogram(sizes)

    if opts.header:
        AGP.print_header(fw, organism="Medicago truncatula",
                         taxid=3880, source="J. Craig Venter Institute")

    if merge:
        for ob, bb in groupby(data, lambda x: x.object):
            for i, b in enumerate(bb):
                b.part_number = i + 1
                print >> fw, b
        return merged_agpfile


def tidy(args):
    """
    %prog tidy agpfile componentfasta

    Given an agp file, run through the following steps:
    1. Trim components with dangling N's
    2. Merge adjacent gaps
    3. Trim gaps at the end of an object
    4. Reindex the agp

    Final output is in `.tidy.agp`.
    """
    p = OptionParser(tidy.__doc__)
    p.add_option("--nogaps", default=False, action="store_true",
                 help="Remove all gap lines [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) != 2:
        sys.exit(p.print_help())

    agpfile, componentfasta = args
    originalagpfile = agpfile

    # Step 1: Trim terminal Ns
    tmpfasta = "tmp.fasta"
    trimmed_agpfile = build([agpfile, componentfasta, tmpfasta,
                             "--newagp", "--novalidate"])
    os.remove(tmpfasta)
    agpfile = trimmed_agpfile
    agpfile = reindex([agpfile, "--inplace"])

    # Step 2: Merge adjacent gaps
    merged_agpfile = gaps([agpfile, "--merge"])
    os.remove(agpfile)

    # Step 3: Trim gaps at the end of object
    agpfile = merged_agpfile
    agp = AGP(agpfile)
    newagpfile = agpfile.replace(".agp", ".fixed.agp")
    fw = open(newagpfile, "w")
    for object, a in groupby(agp, key=lambda x: x.object):
        a = list(a)
        if a[0].is_gap:
            g, a = a[0], a[1:]
            logging.debug("Trim beginning Ns({0}) of {1}".\
                          format(g.gap_length, object))
        if a and a[-1].is_gap:
            a, g = a[:-1], a[-1]
            logging.debug("Trim trailing Ns({0}) of {1}".\
                          format(g.gap_length, object))
        print >> fw, "\n".join(str(x) for x in a)
    fw.close()
    os.remove(agpfile)

    # Step 4: Final reindex
    agpfile = newagpfile
    reindex_opts = [agpfile, "--inplace"]
    if opts.nogaps:
        reindex_opts += ["--nogaps"]
    agpfile = reindex(reindex_opts)

    tidyagpfile = originalagpfile.replace(".agp", ".tidy.agp")
    shutil.move(agpfile, tidyagpfile)

    logging.debug("File written to `{0}`.".format(tidyagpfile))
    return tidyagpfile


def build(args):
    """
    %prog build agpfile componentfasta targetfasta

    Build targetfasta based on info from agpfile
    """
    p = OptionParser(build.__doc__)
    p.add_option("--newagp", dest="newagp", default=False, action="store_true",
            help="Check components to trim dangling N's [default: %default]")
    p.add_option("--novalidate", dest="novalidate", default=False,
            action="store_true",
            help="Don't validate the agpfile [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) != 3:
        sys.exit(not p.print_help())

    agpfile, componentfasta, targetfasta = args
    validate = not opts.novalidate

    if opts.newagp:
        assert agpfile.endswith(".agp")
        newagpfile = agpfile.replace(".agp", ".trimmed.agp")
        newagp = open(newagpfile, "w")
    else:
        newagpfile = None
        newagp = None

    agp = AGP(agpfile, validate=validate, sorted=True)
    agp.build_all(componentfasta=componentfasta, targetfasta=targetfasta,
            newagp=newagp)
    logging.debug("Target fasta written to `{0}`.".format(targetfasta))

    return newagpfile


def validate(args):
    """
    %prog validate agpfile componentfasta targetfasta

    validate consistency between agpfile and targetfasta
    """
    p = OptionParser(validate.__doc__)

    opts, args = p.parse_args(args)

    try:
        agpfile, componentfasta, targetfasta = args
    except Exception, e:
        sys.exit(p.print_help())

    agp = AGP(agpfile)
    build = Fasta(targetfasta)
    bacs = Fasta(componentfasta, index=False)

    # go through this line by line
    for aline in agp:
        try:
            build_seq = build.sequence(dict(chr=aline.object,
                start=aline.object_beg, stop=aline.object_end))

            if aline.is_gap:
                assert build_seq.upper() == aline.gap_length * 'N', \
                    "gap mismatch: %s" % aline
            else:
                bac_seq = bacs.sequence(dict(chr=aline.component_id,
                    start=aline.component_beg, stop=aline.component_end,
                    strand=aline.orientation))

                assert build_seq.upper() == bac_seq.upper(), \
                        "sequence mismatch: %s" % aline

            logging.debug("%s:%d-%d verified" % (aline.object,
                aline.object_beg, aline.object_end))

        except Exception as e:
            logging.error(e)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
MAF format specification:
<http://genome.ucsc.edu/FAQ/FAQformat#format5>
"""

import sys

from bx import interval_index_file
from bx.align import maf

from jcvi.formats.base import BaseFile
from jcvi.apps.base import OptionParser, ActionDispatcher, need_update
from jcvi.apps.lastz import blastz_score_to_ncbi_expectation, \
            blastz_score_to_ncbi_bits


class Maf (BaseFile, dict):

    def __init__(self, filename, index=False):
        super(Maf, self).__init__(filename)

        indexfile = filename + ".idx"
        if index:
            if need_update(filename, indexfile):
                self.build_index(filename, indexfile)

            self.index = maf.Index(filename, indexfile)

        fp = open(filename)
        self.reader = maf.Reader(fp)

    def build_index(self, filename, indexfile):
        """
        Recipe from Brad Chapman's blog
        <http://bcbio.wordpress.com/2009/07/26/sorting-genomic-alignments-using-python/>
        """
        indexes = interval_index_file.Indexes()
        in_handle = open(filename)

        reader = maf.Reader(in_handle)
        while True:
            pos = reader.file.tell()
            rec = reader.next()
            if rec is None:
                break
            for c in rec.components:
                indexes.add(c.src, c.forward_strand_start,
                        c.forward_strand_end, pos, max=c.src_size )

        index_handle = open(indexfile, "w")
        indexes.write(index_handle)
        index_handle.close()


def main():

    actions = (
        ('bed', 'convert MAF to BED format'),
        ('blast', 'convert MAF to BLAST tabular format'),
            )
    p = ActionDispatcher(actions)
    p.dispatch(globals())


def bed(args):
    """
    %prog bed maffiles > out.bed

    Convert a folder of maf alignments to the bed features
    then useful to check coverage, etc.
    """
    p = OptionParser(bed.__doc__)

    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(p.print_help())

    flist = args
    prefix = flist[0].split(".")[0]

    j = 0
    for f in flist:
        reader = Maf(f).reader
        for rec in reader:
            a, b = rec.components

            for a, tag in zip((a, b), "ab"):
                name = "{0}_{1:07d}{2}".format(prefix, j, tag)
                print "\t".join(str(x) for x in (a.src, a.forward_strand_start, \
                        a.forward_strand_end, name))

            j += 1


def alignment_details(a, b):
    nmatch = 0
    nmismatch = 0
    ngaps = 0

    assert len(a) == len(b)
    l = len(a)

    for i in range(l):
        if a[i] == b[i]:
            nmatch += 1
        elif a[i] == "-" or b[i] == "-":
            ngaps += 1
        else:
            nmismatch += 1

    pctid = 100. * nmatch / l
    return pctid, nmismatch, ngaps


def maf_to_blast8(f):
    reader = Maf(f).reader
    for rec in reader:
        a, b = rec.components
        query = a.src
        subject = b.src
        qstart = a.forward_strand_start
        qstop = a.forward_strand_end
        sstart = b.forward_strand_start
        sstop = b.forward_strand_end
        score = rec.score

        evalue = blastz_score_to_ncbi_expectation(score)
        score = blastz_score_to_ncbi_bits(score)
        evalue, score = "{0:.2g}".format(evalue), "{0:.1f}".format(score)
        hitlen = len(a.text)

        pctid, nmismatch, ngaps = alignment_details(a.text, b.text)
        print "\t".join(str(x) for x in (query, subject, pctid, hitlen,
            nmismatch, ngaps, qstart, qstop, sstart, sstop,
            evalue, score))


def blast(args):
    '''
    %prog blast maffiles > out.blast

    From a folder of .maf files, generate .blast file with tabular format.
    '''
    p = OptionParser(blast.__doc__)
    opts, args = p.parse_args(args)

    if len(args) == 0:
        sys.exit(p.print_help())

    flist = args

    for f in flist:
        maf_to_blast8(f)


if __name__ == '__main__':
    main()



"""
Wrapper for calling Bio.Entrez tools to get the sequence from a list of IDs
"""

import os.path as op
import sys
import time
import logging
import urllib2

from Bio import Entrez, SeqIO

from jcvi.formats.base import must_open
from jcvi.formats.fasta import print_first_difference
from jcvi.utils.cbook import tile
from jcvi.utils.iter import grouper
from jcvi.apps.console import green
from jcvi.apps.base import OptionParser, ActionDispatcher, get_email_address, \
            mkdir, ls_ftp, download


myEmail = get_email_address(user=True)
Entrez.email = myEmail


def batch_taxonomy(list_of_taxids):
    """
    Convert list of taxids to Latin names
    """
    for taxid in list_of_taxids:
        handle = Entrez.efetch(db='Taxonomy', id=taxid, retmode="xml")
        records = Entrez.read(handle)
        yield records[0]["ScientificName"]


def batch_taxids(list_of_names):
    """
    Opposite of batch_taxonomy():

    Convert list of Latin names to taxids
    """
    for name in list_of_names:
        handle = Entrez.esearch(db='Taxonomy', term=name, retmode="xml")
        records = Entrez.read(handle)
        yield records["IdList"][0]


def batch_entrez(list_of_terms, db="nuccore", retmax=1, rettype="fasta",
            batchsize=1, email=myEmail):
    """
    Retrieve multiple rather than a single record
    """

    for term in list_of_terms:

        logging.debug("Search term %s" % term)
        success = False
        ids = None
        if not term:
            continue

        while not success:
            try:
                search_handle = Entrez.esearch(db=db, retmax=retmax, term=term)
                rec = Entrez.read(search_handle)
                success = True
                ids = rec["IdList"]
            except (urllib2.HTTPError, urllib2.URLError,
                    RuntimeError, KeyError) as e:
                logging.error(e)
                logging.debug("wait 5 seconds to reconnect...")
                time.sleep(5)

        if not ids:
            logging.error("term {0} not found".format(term))
            continue

        assert ids
        nids = len(ids)
        if nids > 1:
            logging.debug("A total of {0} results found.".format(nids))

        if batchsize != 1:
            logging.debug("Use a batch size of {0}.".format(batchsize))

        ids = list(grouper(ids, batchsize))

        for id in ids:
            id = [x for x in id if x]
            size = len(id)
            id = ",".join(id)

            success = False
            while not success:
                try:
                    fetch_handle = Entrez.efetch(db=db, id=id, rettype=rettype,
                            email=email)
                    success = True
                except (urllib2.HTTPError, urllib2.URLError,
                        RuntimeError) as e:
                    logging.error(e)
                    logging.debug("wait 5 seconds to reconnect...")
                    time.sleep(5)

            yield id, size, term, fetch_handle


def main():

    actions = (
        ('fetch', 'fetch records from a list of GenBank accessions'),
        ('bisect', 'determine the version of the accession'),
        ('phytozome', 'retrieve genomes and annotations from phytozome'),
        ('ensembl', 'retrieve genomes and annotations from ensembl'),
        )
    p = ActionDispatcher(actions)
    p.dispatch(globals())


def ensembl(args):
    """
    %prog ensembl species

    Retrieve genomes and annotations from ensembl FTP. Available species
    listed below. Use comma to give a list of species to download. For example:

    $ %prog ensembl danio_rerio,gasterosteus_aculeatus
    """
    p = OptionParser(ensembl.__doc__)
    p.add_option("--version", default="75",
                 help="Ensembl version [default: %default]")
    opts, args = p.parse_args(args)

    version = opts.version
    url = "ftp://ftp.ensembl.org/pub/release-{0}/".format(version)
    fasta_url = url + "fasta/"

    valid_species = [x for x in ls_ftp(fasta_url) if "." not in x]
    doc = "\n".join((ensembl.__doc__, tile(valid_species)))
    p.set_usage(doc)

    if len(args) != 1:
        sys.exit(not p.print_help())

    species, = args
    species = species.split(",")
    for s in species:
        download_species_ensembl(s, valid_species, url)


def download_species_ensembl(species, valid_species, url):
    from os.path import join as urljoin

    assert species in valid_species, \
            "{0} is not in the species list".format(species)

    # We want to download assembly and annotation for given species
    ann_url = urljoin(url, "gtf/{0}".format(species))
    cds_url = urljoin(url, "fasta/{0}/cds".format(species))

    for u in (ann_url, cds_url):
        valid_files = [x for x in ls_ftp(u) if x.endswith(".gz")]
        for f in valid_files:
            f = urljoin(u, f)
            download(f)


def phytozome(args):
    """
    %prog phytozome species

    Retrieve genomes and annotations from phytozome FTP. Available species
    listed below. Use comma to give a list of species to download. For example:

    $ %prog phytozome Athaliana,Vvinifera,Osativa,Sbicolor,Slycopersicum
    """
    p = OptionParser(phytozome.__doc__)
    p.add_option("--version", default="9.0",
                 help="Phytozome version [default: %default]")
    p.add_option("--assembly", default=False, action="store_true",
                 help="Download assembly [default: %default]")
    opts, args = p.parse_args(args)

    url = "ftp://ftp.jgi-psf.org/pub/compgen/phytozome/v{0}/".\
                    format(opts.version)
    valid_species = [x for x in ls_ftp(url) if "." not in x]

    doc = "\n".join((phytozome.__doc__, tile(valid_species)))
    p.set_usage(doc)

    if len(args) != 1:
        sys.exit(not p.print_help())

    species, = args
    species = species.split(",")
    for s in species:
        download_species_phytozome(s, valid_species, url, assembly=opts.assembly)


def download_species_phytozome(species, valid_species, url, assembly=False):
    from os.path import join as urljoin

    assert species in valid_species, \
            "{0} is not in the species list".format(species)

    # We want to download assembly and annotation for given species
    surl = urljoin(url, species)
    contents = [x for x in ls_ftp(surl) if x.endswith("_readme.txt")]
    magic = contents[0].split("_")[1]  # Get the magic number
    logging.debug("Found magic number for {0}: {1}".format(species, magic))

    pf = "{0}_{1}".format(species, magic)
    asm_url = urljoin(surl, "assembly/{0}.fa.gz".format(pf))
    ann_url = urljoin(surl, "annotation/{0}_gene.gff3.gz".format(pf))
    cds_url = urljoin(surl, "annotation/{0}_cds.fa.gz".format(pf))
    if assembly:
        download(asm_url)
    for u in (ann_url, cds_url):
        download(u)


def get_first_rec(fastafile):
    """
    Returns the first record in the fastafile
    """
    f = list(SeqIO.parse(fastafile, "fasta"))

    if len(f) > 1:
        logging.debug("{0} records found in {1}, using the first one".\
                format(len(f), fastafile))

    return f[0]


def bisect(args):
    """
    %prog bisect acc accession.fasta

    determine the version of the accession, based on a fasta file.
    This proceeds by a sequential search from xxxx.1 to the latest record.
    """
    p = OptionParser(bisect.__doc__)
    p.set_email()

    opts, args = p.parse_args(args)

    if len(args) != 2:
        sys.exit(not p.print_help())

    acc, fastafile = args
    arec = get_first_rec(fastafile)

    valid = None
    for i in range(1, 100):
        term = "%s.%d" % (acc, i)
        try:
            query = list(batch_entrez([term], email=opts.email))
        except AssertionError as e:
            logging.debug("no records found for %s. terminating." % term)
            return

        id, term, handle = query[0]
        brec = SeqIO.parse(handle, "fasta").next()

        match = print_first_difference(arec, brec, ignore_case=True,
                ignore_N=True, rc=True)
        if match:
            valid = term
            break

    if valid:
        print
        print green("%s matches the sequence in `%s`" % (valid, fastafile))


def fetch(args):
    """
    %prog fetch <filename|term>

    `filename` contains a list of terms to search. Or just one term. If the
    results are small in size, e.g. "--format=acc", use "--batchsize=100" to speed
    the download.
    """
    p = OptionParser(fetch.__doc__)

    allowed_databases = {"fasta": ["genome", "nuccore", "nucgss", "protein", "nucest"],
                         "asn.1": ["genome", "nuccore", "nucgss", "protein"],
                         "gb"   : ["genome", "nuccore", "nucgss"],
                         "est"  : ["nucest"],
                         "gss"  : ["nucgss"],
                         "acc"  : ["nuccore"],
                        }

    valid_formats = tuple(allowed_databases.keys())
    valid_databases = ("genome", "nuccore", "nucest", "nucgss", "protein")

    p.add_option("--noversion", dest="noversion",
            default=False, action="store_true",
            help="Remove trailing accession versions")
    p.add_option("--format", default="fasta", choices=valid_formats,
            help="download format [default: %default]")
    p.add_option("--database", default="nuccore", choices=valid_databases,
            help="search database [default: %default]")
    p.add_option("--retmax", default=1000000, type="int",
            help="how many results to return [default: %default]")
    p.add_option("--skipcheck", default=False, action="store_true",
            help="turn off prompt to check file existence [default: %default]")
    p.add_option("--batchsize", default=500, type="int",
            help="download the results in batch for speed-up [default: %default]")
    p.add_option("--outdir", default=None,
            help="output directory, with accession number as filename")
    p.add_option("--outprefix", default="out",
            help="output file name prefix [default: %default]")
    p.set_email()
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(p.print_help())

    filename, = args
    if op.exists(filename):
        pf = filename.rsplit(".", 1)[0]
        list_of_terms = [row.strip() for row in open(filename)]
        if opts.noversion:
            list_of_terms = [x.rsplit(".", 1)[0] for x in list_of_terms]
    else:
        pf = filename
        # the filename is the search term
        list_of_terms = [filename.strip()]

    fmt = opts.format
    database = opts.database
    batchsize = opts.batchsize

    assert database in allowed_databases[fmt], \
        "For output format '{0}', allowed databases are: {1}".\
        format(fmt, allowed_databases[fmt])
    assert batchsize >= 1, "batchsize must >= 1"

    if " " in pf:
        pf = opts.outprefix

    outfile = "{0}.{1}".format(pf, fmt)

    outdir = opts.outdir
    if outdir:
        mkdir(outdir)

    # If noprompt, will not check file existence
    if not outdir:
        fw = must_open(outfile, "w", checkexists=True, \
                skipcheck=opts.skipcheck)
        if fw is None:
            return

    seen = set()
    totalsize = 0
    for id, size, term, handle in batch_entrez(list_of_terms, retmax=opts.retmax, \
                                 rettype=fmt, db=database, batchsize=batchsize, \
                                 email=opts.email):
        if outdir:
            outfile = op.join(outdir, "{0}.{1}".format(term, fmt))
            fw = must_open(outfile, "w", checkexists=True, \
                    skipcheck=opts.skipcheck)
            if fw is None:
                continue

        rec = handle.read()
        if id in seen:
            logging.error("Duplicate key ({0}) found".format(rec))
            continue

        totalsize += size
        print >> fw, rec
        print >> fw

        seen.add(id)

    if seen:
        print >> sys.stderr, "A total of {0} {1} records downloaded.".\
                format(totalsize, fmt.upper())

    return outfile


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os.path as op
import sys
import logging

from multiprocessing import Lock

from jcvi.formats.base import must_open, split
from jcvi.apps.grid import Jobs
from jcvi.apps.align import run_formatdb
from jcvi.apps.base import OptionParser, Popen


def blastplus(k, n, out_fh, cmd, query, lock):

    cmd += " -query {0}".format(query)
    proc = Popen(cmd)

    logging.debug("job <%d> started: %s" % (proc.pid, cmd))
    for row in proc.stdout:
        if row[0] == '#':
            continue
        lock.acquire()
        out_fh.write(row)
        out_fh.flush()
        lock.release()
    logging.debug("job <%d> finished" % proc.pid)


def main():
    """
    %prog database.fa query.fa [options]

    Wrapper for NCBI BLAST+.
    """
    p = OptionParser(main.__doc__)

    p.add_option("--format", default=" \'6 qseqid sseqid pident length " \
            "mismatch gapopen qstart qend sstart send evalue bitscore\' ",
            help="0-11, learn more with \"blastp -help\". [default: %default]")
    p.add_option("--path", dest="blast_path", default=None,
            help="specify BLAST+ path including the program name")
    p.add_option("--prog", dest="blast_program", default="blastp",
            help="specify BLAST+ program to use. See complete list here: " \
            "http://www.ncbi.nlm.nih.gov/books/NBK52640/#chapter1.Installation"
            " [default: %default]")
    p.set_align(evalue=.01)
    p.add_option("--best", default=1, type="int",
            help="Only look for best N hits [default: %default]")

    p.set_cpus()
    p.set_params()
    p.set_outfile()
    opts, args = p.parse_args()

    if len(args) != 2 or opts.blast_program is None:
        sys.exit(not p.print_help())

    bfasta_fn, afasta_fn = args
    for fn in (afasta_fn, bfasta_fn):
        assert op.exists(fn)

    afasta_fn = op.abspath(afasta_fn)
    bfasta_fn = op.abspath(bfasta_fn)
    out_fh = must_open(opts.outfile, "w")

    extra = opts.extra
    blast_path = opts.blast_path
    blast_program = opts.blast_program

    blast_bin = blast_path or blast_program
    if op.basename(blast_bin) != blast_program:
        blast_bin = "".join([blast_bin, "/", blast_program])

    cpus = opts.cpus
    if cpus > 1:
        logging.debug("Dispatch job to %d cpus" % cpus)
        outdir = "outdir"
        fs = split([afasta_fn, outdir, str(cpus)])
        queries = fs.names
    else:
        queries = [afasta_fn]

    dbtype = "prot" if op.basename(blast_bin) in ["blastp", "blastx"] \
        else "nucl"

    db = bfasta_fn
    if dbtype == "prot":
        nin = db + ".pin"
    else:
        nin = db + ".nin"
        nin00 = db + ".00.nin"
        nin = nin00 if op.exists(nin00) else (db + ".nin")

    run_formatdb(infile=db, outfile=nin, dbtype=dbtype)

    lock = Lock()

    blastplus_template = "{0} -db {1} -outfmt {2}"
    blast_cmd = blastplus_template.format(blast_bin, bfasta_fn, opts.format)
    blast_cmd += " -evalue {0} -max_target_seqs {1}".\
        format(opts.evalue, opts.best)
    if extra:
        blast_cmd += " " + extra.strip()

    args = [(k + 1, cpus, out_fh, blast_cmd, query, lock) \
                for k, query in zip(range(cpus), queries)]
    g = Jobs(target=blastplus, args=args)
    g.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
Adapted from Seth M. Morton's natsort library:
<https://github.com/SethMMorton/natsort>

Here are a collection of examples of how this module can be used.
See the README or the natsort homepage for more details.

    >>> a = ['a2', 'a5', 'a9', 'a1', 'a4', 'a10', 'a6']
    >>> sorted(a)
    ['a1', 'a10', 'a2', 'a4', 'a5', 'a6', 'a9']
    >>> natsorted(a)
    ['a1', 'a2', 'a4', 'a5', 'a6', 'a9', 'a10']

Here is an example demonstrating how different options sort the same list.

    >>> a = ['a50', 'a51.', 'a50.31', 'a50.4', 'a5.034e1', 'a50.300']
    >>> sorted(a)
    ['a5.034e1', 'a50', 'a50.300', 'a50.31', 'a50.4', 'a51.']
    >>> natsorted(a)
    ['a50', 'a50.300', 'a50.31', 'a5.034e1', 'a50.4', 'a51.']
    >>> natsorted(a, number_type=float, exp=False)
    ['a5.034e1', 'a50', 'a50.300', 'a50.31', 'a50.4', 'a51.']
    >>> natsorted(a, number_type=int)
    ['a5.034e1', 'a50', 'a50.4', 'a50.31', 'a50.300', 'a51.']
    >>> natsorted(a, number_type=None)
    ['a5.034e1', 'a50', 'a50.4', 'a50.31', 'a50.300', 'a51.']

This demonstrates the signed option.  It can account for negative and positive signs.
Turning it off treats the '+' or '-' as part of the string.

    >>> a = ['a-5', 'a7', 'a+2']
    >>> sorted(a)
    ['a+2', 'a-5', 'a7']
    >>> natsorted(a) # signed=True is default, -5 comes first on the number line
    ['a-5', 'a+2', 'a7']
    >>> natsorted(a, signed=False) # 'a' comes before 'a+', which is before 'a-'
    ['a7', 'a+2', 'a-5']

Sorting version numbers is best with 'number_type=None'.  That is a shortcut
for 'number_type=int, signed=False'

    >>> a = ['1.9.9a', '1.11', '1.9.9b', '1.11.4', '1.10.1']
    >>> sorted(a)
    ['1.10.1', '1.11', '1.11.4', '1.9.9a', '1.9.9b']
    >>> natsorted(a)
    ['1.10.1', '1.11', '1.11.4', '1.9.9a', '1.9.9b']
    >>> natsorted(a, number_type=None)
    ['1.9.9a', '1.9.9b', '1.10.1', '1.11', '1.11.4']

You can mix types with natsorted.  This can get around the new
'unorderable types' issue with Python 3.

    >>> import sys
    >>> a = [6, 4.5, '7', '2.5', 'a']
    >>> natsorted(a)
    ['2.5', 4.5, 6, '7', 'a']

natsort will recursively descend into lists of lists so you can sort by the sublist contents.

    >>> data = [['a1', 'a5'], ['a1', 'a40'], ['a10', 'a1'], ['a2', 'a5']]
    >>> sorted(data)
    [['a1', 'a40'], ['a1', 'a5'], ['a10', 'a1'], ['a2', 'a5']]
    >>> natsorted(data)
    [['a1', 'a5'], ['a1', 'a40'], ['a2', 'a5'], ['a10', 'a1']]

"""


import re


# The regex that locates floats
float_sign_exp_re = re.compile(r'([-+]?\d*\.?\d+(?:[eE][-+]?\d+)?)')
float_nosign_exp_re = re.compile(r'(\d*\.?\d+(?:[eE][-+]?\d+)?)')
float_sign_noexp_re = re.compile(r'([-+]?\d*\.?\d+)')
float_nosign_noexp_re = re.compile(r'(\d*\.?\d+)')
# Integer regexes
int_nosign_re = re.compile(r'(\d+)')
int_sign_re = re.compile(r'([-+]?\d+)')
# This dict will help select the correct regex and number conversion function.
regex_and_num_function_chooser = {
    (float, True,  True)  : (float_sign_exp_re,     float),
    (float, True,  False) : (float_sign_noexp_re,   float),
    (float, False, True)  : (float_nosign_exp_re,   float),
    (float, False, False) : (float_nosign_noexp_re, float),
    (int,   True,  True)  : (int_sign_re,   int),
    (int,   True,  False) : (int_sign_re,   int),
    (int,   False, True)  : (int_nosign_re, int),
    (int,   False, False) : (int_nosign_re, int),
    (None,  True,  True)  : (int_nosign_re, int),
    (None,  True,  False) : (int_nosign_re, int),
    (None,  False, True)  : (int_nosign_re, int),
    (None,  False, False) : (int_nosign_re, int),
}


def remove_empty(s):
    """\
    Remove empty strings from a list.

        >>> a = ['a', 2, '', 'b', '']
        >>> remove_empty(a)
        ['a', 2, 'b']

    """
    while True:
        try:
            s.remove('')
        except ValueError:
            break
    return s


def _number_finder(s, regex, numconv):
    """Helper to split numbers"""

    # Split.  If there are no splits, return now
    s = regex.split(s)
    if len(s) == 1:
        return tuple(s)

    # Now convert the numbers to numbers, and leave strings as strings
    s = remove_empty(s)
    for i in range(len(s)):
        try:
            s[i] = numconv(s[i])
        except ValueError:
            pass

    # If the list begins with a number, lead with an empty string.
    # This is used to get around the "unorderable types" issue.
    if not isinstance(s[0], basestring):
        return [''] + s
    else:
        return s


def natsort_key(s, number_type=int, signed=False, exp=False):
    """\
    Key to sort strings and numbers naturally, not lexicographically.
    It also has basic support for version numbers.
    For use in passing to the :py:func:`sorted` builtin or
    :py:meth:`sort` attribute of lists.

    Use natsort_key just like any other sorting key.

        >>> a = ['num3', 'num5', 'num2']
        >>> a.sort(key=natsort_key)
        >>> a
        ['num2', 'num3', 'num5']

    Below illustrates how the key works, and how the different options affect sorting.

        >>> natsort_key('a-5.034e1')
        ('a-', 5, '.', 34, 'e', 1)
        >>> natsort_key('a-5.034e1', number_type=float, signed=True, exp=True)
        ('a', -50.34)
        >>> natsort_key('a-5.034e1', number_type=float, signed=True, exp=False)
        ('a', -5.034, 'e', 1.0)
        >>> natsort_key('a-5.034e1', number_type=float, signed=False, exp=True)
        ('a-', 50.34)
        >>> natsort_key('a-5.034e1', number_type=float, signed=False, exp=False)
        ('a-', 5.034, 'e', 1.0)
        >>> natsort_key('a-5.034e1', number_type=int, signed=True)
        ('a', -5, '.', 34, 'e', 1)
        >>> natsort_key('a-5.034e1', number_type=int, signed=False)
        ('a-', 5, '.', 34, 'e', 1)
        >>> natsort_key('a-5.034e1', number_type=int, exp=False)
        ('a-', 5, '.', 34, 'e', 1)
        >>> natsort_key('a-5.034e1', number_type=None)
        ('a-', 5, '.', 34, 'e', 1)

    This is a demonstration of what number_type=None works.

        >>> natsort_key('a-5.034e1', number_type=None) == natsort_key('a-5.034e1', number_type=None, signed=False)
        True
        >>> natsort_key('a-5.034e1', number_type=None) == natsort_key('a-5.034e1', number_type=None, exp=False)
        True
        >>> natsort_key('a-5.034e1', number_type=None) == natsort_key('a-5.034e1', number_type=int, signed=False)
        True

    Iterables are parsed recursively so you can sort lists of lists.

        >>> natsort_key(('a1', 'a10'))
        (('a', 1), ('a', 10))

    Strings that lead with a number get an empty string at the front of the tuple.
    This is designed to get around the "unorderable types" issue.

        >>> natsort_key(('15a', '6'))
        (('', 15, 'a'), ('', 6))

    You can give numbers, too.

        >>> natsort_key(10)
        ('', 10)

    """

    # If we are dealing with non-strings, return now
    if not isinstance(s, basestring):
        if hasattr(s, '__getitem__'):
            return tuple(natsort_key(x) for x in s)
        else:
            return ('', s,)

    # Convert to the proper tuple and return
    inp_options = (number_type, signed, exp)
    args = (s,) + regex_and_num_function_chooser[inp_options]
    try:
        return tuple(_number_finder(*args))
    except KeyError:
        # Report errors properly
        if number_type not in (float, int) or number_type is not None:
            raise ValueError("natsort_key: 'number_type' "
                             "parameter '{0}'' invalid".format(str(number_type)))
        elif signed not in (True, False):
            raise ValueError("natsort_key: 'signed' "
                             "parameter '{0}'' invalid".format(str(signed)))
        elif exp not in (True, False):
            raise ValueError("natsort_key: 'exp' "
                             "parameter '{0}'' invalid".format(str(exp)))


def natsorted(seq, key=lambda x: x, number_type=float, signed=True, exp=True):
    """\
    Sorts a sequence naturally (alphabetically and numerically),
    not lexicographically.

        >>> a = ['num3', 'num5', 'num2']
        >>> natsorted(a)
        ['num2', 'num3', 'num5']
        >>> b = [('a', 'num3'), ('b', 'num5'), ('c', 'num2')]
        >>> from operator import itemgetter
        >>> natsorted(b, key=itemgetter(1))
        [('c', 'num2'), ('a', 'num3'), ('b', 'num5')]

    """
    return sorted(seq, key=lambda x: natsort_key(key(x),
                                                 number_type=number_type,
                                                 signed=signed, exp=exp))


def index_natsorted(seq, key=lambda x: x, number_type=float, signed=True, exp=True):
    """\
    Sorts a sequence naturally, but returns a list of sorted the
    indeces and not the sorted list.

        >>> a = ['num3', 'num5', 'num2']
        >>> b = ['foo', 'bar', 'baz']
        >>> index = index_natsorted(a)
        >>> index
        [2, 0, 1]
        >>> # Sort both lists by the sort order of a
        >>> [a[i] for i in index]
        ['num2', 'num3', 'num5']
        >>> [b[i] for i in index]
        ['baz', 'foo', 'bar']
        >>> c = [('a', 'num3'), ('b', 'num5'), ('c', 'num2')]
        >>> from operator import itemgetter
        >>> index_natsorted(c, key=itemgetter(1))
        [2, 0, 1]

    """
    from operator import itemgetter
    item1 = itemgetter(1)
    # Pair the index and sequence together, then sort by
    index_seq_pair = [[x, key(y)] for x, y in zip(range(len(seq)), seq)]
    index_seq_pair.sort(key=lambda x: natsort_key(item1(x),
                                                  number_type=number_type,
                                                  signed=signed, exp=exp))
    return [x[0] for x in index_seq_pair]


def test():
    from doctest import DocTestSuite
    return DocTestSuite()


# Test this module
if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python

from setuptools import setup, find_packages
from glob import glob


name = "jcvi"
classifiers = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Topic :: Scientific/Engineering :: Bio-Informatics',
    ]

exec(open("__init__.py").read())
setup(
      name=name,
      version=__version__,
      author='Haibao Tang',
      author_email=__email__,
      package_dir={name: '.'},
      packages=[name] + ['.'.join((name, x)) for x in find_packages()],
      include_package_data=True,
      data_files=[(name + '/utils/data', glob("utils/data/*"))],
      classifiers=classifiers,
      zip_safe=False,
      license='BSD',
      url='http://github.com/tanghaibao/jcvi',
      description='Python utility libraries on genome assembly, '\
                  'annotation and comparative genomics',
      long_description=open("README.rst").read(),
      install_requires=['biopython', 'numpy', 'matplotlib']
     )



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
Wrapper for the common graph algorithms.
"""

import sys
import logging

import networkx as nx
from collections import deque
from string import maketrans

from jcvi.utils.iter import pairwise
from jcvi.formats.base import must_open


"""
Bidirectional graph.
"""
dirs = (">", "<")
trans = maketrans("+?-", ">><")


class BiNode (object):

    def __init__(self, v):
        self.v = v
        self.ins = []
        self.outs = []

    def get_next(self, tag="<"):
        """
        This function is tricky and took me a while to figure out.

        The tag specifies the direction where the current edge came from.

         tag     ntag
        ---> V >----> U
             cur      next

        This means the next vertex should follow the outs since this tag is
        inward '<'. Check if there are multiple branches if len(L) == 1, and
        also check if the next it finds has multiple incoming edges though if
        len(B) == 1.
        """
        next, ntag = None, None

        L = self.outs if tag == "<" else self.ins

        if len(L) == 1:
            e, = L
            if e.v1.v == self.v:
                next, ntag = e.v2, e.o2
                ntag = "<" if ntag == ">" else ">"  # Flip tag if on other end
            else:
                next, ntag = e.v1, e.o1

        if next:  # Validate the next vertex
            B = next.ins if ntag == "<" else next.outs
            if len(B) > 1:
                return None, None

        return next, ntag

    def __str__(self):
        return str(self.v)

    __repr__ = __str__


class BiEdge (object):

    def __init__(self, v1, v2, o1, o2, color="black", length=None):

        self.v1 = v1
        self.v2 = v2

        o1 = o1.translate(trans)
        o2 = o2.translate(trans)

        assert o1 in dirs and o2 in dirs
        self.o1 = o1
        self.o2 = o2

        if v1 > v2:
            self.flip()

        self.color = color
        self.length = length

    def __str__(self):
        return "".join(str(x) for x in \
                 (self.v1, self.o1, "--", self.o2, self.v2))

    def flip(self):
        self.v2, self.v1 = self.v1, self.v2
        o1, o2 = self.o1, self.o2
        self.o1 = ">" if o2 == "<" else "<"
        self.o2 = ">" if o1 == "<" else "<"


class BiGraph (object):

    def __init__(self):
        self.nodes = {}
        self.edges = {}

    def __str__(self):
        return "BiGraph with {0} nodes and {1} edges".\
                format(len(self.nodes), len(self.edges))

    def add_node(self, v):
        if v not in self.nodes:
            self.nodes[v] = BiNode(v)

    def add_edge(self, e):
        v1, v2 = e.v1, e.v2

        assert isinstance(e, BiEdge)
        for v in (v1, v2):
            self.add_node(v)
        n1 = self.nodes.get(v1)
        n2 = self.nodes.get(v2)
        l = n1.outs if e.o1 == ">" else n1.ins
        r = n2.ins if e.o2 == ">" else n2.outs
        l.append(e)
        r.append(e)
        e.v1, e.v2 = n1, n2
        self.edges[(v1, v2)] = e

    def get_node(self, v):
        return self.nodes[v]

    def get_edge(self, av, bv):
        flip = False
        if av > bv:
            av, bv = bv, av
            flip = True
        e = self.edges[(av, bv)]
        if flip:
            e.flip()
        return e

    def iter_paths(self):

        discovered = set()
        for v, vv in self.nodes.items():
            if v in discovered:
                continue

            path = deque([vv])

            #print "cur", v
            discovered.add(v)
            prev, ptag = vv.get_next(tag=">")
            while prev:
                #print prev, ptag
                if prev.v in discovered:
                    break
                path.appendleft(prev)
                discovered.add(prev.v)
                prev, ptag = prev.get_next(tag=ptag)

            next, ntag = vv.get_next(tag="<")
            while next:
                #print next, ntag
                if next.v in discovered:
                    break
                path.append(next)
                discovered.add(next.v)
                next, ntag = next.get_next(tag=ntag)

            #discovered |= set(x.v for x in path)
            yield path

    def path(self, path, flip=False):
        oo = []
        if len(path) == 1:
            m = "Singleton {0}".format(path[0])
            oo.append((path[0].v, True))
            return m, oo

        edges = []
        for a, b in pairwise(path):
            av, bv = a.v, b.v
            e = self.get_edge(av, bv)

            if not oo:  # First edge imports two nodes
                oo.append((e.v1.v, e.o1 == ">"))
            last = oo[-1]
            assert last == (e.v1.v, e.o1 == ">")
            oo.append((e.v2.v, e.o2 == ">"))

            if flip:
                se = str(e)
                e.flip()
            else:
                se = str(e)
            edges.append(se)

        return "|".join(edges), oo

    def read(self, filename, color="black"):
        fp = open(filename)
        nedges = 0
        for row in fp:
            a, b = row.strip().split("--")
            oa = a[-1]
            ob = b[0]
            a, b = a.strip("<>"), b.strip("<>")
            self.add_edge(BiEdge(a, b, oa, ob, color=color))
            nedges += 1
        logging.debug("A total of {0} edges imported from `{1}` (color={2}).".
                      format(nedges, filename, color))

    def write(self, filename="stdout"):

        fw = must_open(filename, "w")
        for e in self.edges.values():
            print >> fw, e
        logging.debug("Graph written to `{0}`.".format(filename))

    def draw(self, pngfile, dpi=96, verbose=False, namestart=0,
                nodehighlight=None, prog="circo"):
        import pygraphviz as pgv

        G = pgv.AGraph()
        for e in self.edges.values():
            arrowhead = (e.o1 == ">")
            arrowtail = (e.o2 == "<")
            if e.o1 != e.o2:  # Not sure why this is necessary
                arrowhead = not arrowhead
                arrowtail = not arrowtail
            arrowhead = "normal" if arrowhead else "inv"
            arrowtail = "normal" if arrowtail else "inv"
            v1, v2 = e.v1, e.v2
            v1, v2 = str(v1)[namestart:], str(v2)[namestart:]
            G.add_edge(v1, v2, color=e.color,
                       arrowhead=arrowhead, arrowtail=arrowtail)

        if nodehighlight:
            for n in nodehighlight:
                n = n[namestart:]
                n = G.get_node(n)
                n.attr["shape"] = "box"

        G.graph_attr.update(dpi=str(dpi))
        if verbose:
            G.write(sys.stderr)
        G.draw(pngfile, prog=prog)
        logging.debug("Graph written to `{0}`.".format(pngfile))

    def get_next(self, node, tag="<"):
        return self.get_node(node).get_next(tag)

    def get_path(self, n1, n2, tag="<"):
        # return all intermediate nodes on path n1 -> n2
        path = deque()
        next, ntag = self.get_next(n1, tag=tag)
        while next:
            if next.v == n2:
                return path
            path.append((next, ntag))
            next, ntag = next.get_next(tag=ntag)
        return path if n2 is None else None


def bigraph_test():
    g = BiGraph()
    g.add_edge(BiEdge(1, 2, ">", "<"))
    g.add_edge(BiEdge(2, 3, "<", "<", color="red"))
    g.add_edge(BiEdge(2, 3, ">", ">", color="blue"))
    g.add_edge(BiEdge(5, 3, ">", ">"))
    g.add_edge(BiEdge(4, 3, "<", ">"))
    g.add_edge(BiEdge(4, 6, ">", ">"))
    g.add_edge(BiEdge(7, 1, ">", ">"))
    g.add_edge(BiEdge(7, 5, "<", ">"))
    g.add_edge(BiEdge(8, 6, ">", "<"))
    print g
    g.write()
    for path in g.iter_paths():
        p, oo = g.path(path)
        print p
        print oo

    #g.draw("demo.png", verbose=True)


def update_weight(G, a, b, w):
    if G.has_edge(a, b):  # Parallel edges found!
        G[a][b]['weight'] += w
    else:
        G.add_edge(a, b, weight=w)


def make_paths(paths, weights=None):
    """
    Zip together paths. Called by merge_paths().
    """
    npaths = len(paths)
    weights = weights or [1] * npaths
    assert len(paths) == len(weights)

    G = nx.DiGraph()
    for path, w in zip(paths, weights):
        for a, b in pairwise(path):
            update_weight(G, a, b, w)
    return G


def reduce_paths(G):
    """
    Make graph into a directed acyclic graph (DAG).
    """
    from jcvi.algorithms.lpsolve import min_feedback_arc_set

    while not nx.is_directed_acyclic_graph(G):
        edges = []
        for a, b, w in G.edges_iter(data=True):
            w = w['weight']
            edges.append((a, b, w))
        mf, mf_score = min_feedback_arc_set(edges)
        for a, b, w in mf:
            G.remove_edge(a, b)

    assert nx.is_directed_acyclic_graph(G)
    G = transitive_reduction(G)
    return G


def draw_graph(G, pngfile, prog="dot"):
    G = nx.to_agraph(G)
    G.draw(pngfile, prog=prog)
    logging.debug("Graph written to `{0}`.".format(pngfile))


def transitive_reduction(G):
    """
    Returns a transitive reduction of a graph.  The original graph
    is not modified.

    A transitive reduction H of G has a path from x to y if and
    only if there was a path from x to y in G.  Deleting any edge
    of H destroys this property.  A transitive reduction is not
    unique in general.  A transitive reduction has the same
    transitive closure as the original graph.

    A transitive reduction of a complete graph is a tree.  A
    transitive reduction of a tree is itself.

    >>> G = nx.DiGraph([(1, 2), (1, 3), (2, 3), (2, 4), (3, 4)])
    >>> H = transitive_reduction(G)
    >>> H.edges()
    [(1, 2), (2, 3), (3, 4)]
    """
    H = G.copy()
    for a, b, w in G.edges_iter(data=True):
        # Try deleting the edge, see if we still have a path
        # between the vertices
        H.remove_edge(a, b)
        if not nx.has_path(H, a, b):  # we shouldn't have deleted it
            H.add_edge(a, b, w)
    return H


def merge_paths(paths, weights=None):
    """
    Zip together sorted lists.

    >>> paths = [[1, 2, 3], [1, 3, 4], [2, 4, 5]]
    >>> G = merge_paths(paths)
    >>> nx.topological_sort(G)
    [1, 2, 3, 4, 5]
    >>> paths = [[1, 2, 3, 4], [1, 2, 3, 2, 4]]
    >>> G = merge_paths(paths, weights=(1, 2))
    >>> nx.topological_sort(G)
    [1, 2, 3, 4]
    """
    G = make_paths(paths, weights=weights)
    G = reduce_paths(G)
    return G


def longest_path_weighted_nodes(G, source, target, weights=None):
    """
    The longest path problem is the problem of finding a simple path of maximum
    length in a given graph. While for general graph, this problem is NP-hard,
    but if G is a directed acyclic graph (DAG), longest paths in G can be found
    in linear time with dynamic programming.

    >>> G = nx.DiGraph([(1, 2), (1, 3), (2, "M"), (3, "M")])
    >>> longest_path_weighted_nodes(G, 1, "M", weights={1: 1, 2: 1, 3: 2, "M": 1})
    ([1, 3, 'M'], 4)
    """
    assert nx.is_directed_acyclic_graph(G)

    tree = nx.topological_sort(G)
    node_to_index = dict((t, i) for i, t in enumerate(tree))

    nnodes = len(tree)
    weights = [weights.get(x, 1) for x in tree] if weights else [1] * nnodes
    score, fromc = weights[:], [-1] * nnodes
    si = node_to_index[source]
    ti = node_to_index[target]
    for a in tree[si: ti]:
        ai = node_to_index[a]
        for b, w in G[a].items():
            bi = node_to_index[b]
            w = w.get('weight', 1)
            d = score[ai] + weights[bi] * w  # Favor heavier edges
            if d <= score[bi]:
                continue
            score[bi] = d  # Update longest distance so far
            fromc[bi] = ai

    # Backtracking
    path = []
    while ti != -1:
        path.append(ti)
        ti = fromc[ti]

    path = [tree[x] for x in path[::-1]]
    return path, score[ti]


if __name__ == '__main__':
    import doctest
    doctest.testmod()
    #bigraph_test()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
Convert common output files from gene prediction softwares into gff3 format.

Similar to the utilities in DAWGPAWS.
<http://dawgpaws.sourceforge.net/man.html>
"""

import os
import sys
import logging
import re

from itertools import groupby, product

from jcvi.formats.bed import Bed, BedLine, sort
from jcvi.formats.base import SetFile, must_open, get_number
from jcvi.apps.base import OptionParser, OptionGroup, ActionDispatcher, \
            need_update, popen, sh


FRAME, RETAIN, OVERLAP, NEW = "FRAME", "RETAIN", "OVERLAP", "NEW"
PRIORITY = (FRAME, RETAIN, OVERLAP, NEW)
new_id_pat = re.compile(r"^\d+\.[cemtx]+\S+")


class Stride (object):
    """
    Allows four basic strides:

        0 10
       0 5 10
      0 3 7 10
     0 2 5 8 10

    We have main parameters, # we need, # available go through all possible
    numbers excluding everything in black.
    """
    def __init__(self, needed, available):
        configurations = ("0", "05", "037", "0258")
        nneeded = len(needed)
        self.conf = None
        self.available = None
        for c in configurations:
            a = [x for x in available if str(x)[-1] in c]
            if len(a) >= nneeded:
                self.conf = c
                self.available = a
                break


class NameRegister (object):

    def __init__(self):
        self.black = set()
        self.gaps = []

    def get_blacklist(self, filename):
        black = SetFile(filename)
        black = set(atg_name(x) for x in black)
        self.black.update(black)

    def get_gaps(self, filename):
        self.gapfile = filename

    def allocate(self, info, chr, start_id, end_id, id_table):

        start_bp = info[0].start
        end_bp = info[-1].end

        current_chr = get_number(chr)
        needed = info
        assert end_id > start_id, \
            "end ({0}) > start ({1})".format(end_id, start_id)

        spots = end_id - start_id - 1
        available = [x for x in xrange(start_id + 1, end_id) if
                            (current_chr, x) not in self.black]

        message = "chr{0} need {1} ids, has {2} spots ({3} available)".\
                format(current_chr, len(needed), spots, len(available))

        start_gene = gene_name(current_chr, start_id)
        end_gene = gene_name(current_chr, end_id)
        message += " between {0} - {1}\n".format(start_gene, end_gene)

        assert end_bp > start_bp

        b = "\t".join(str(x) for x in (chr, start_bp - 1, end_bp))
        cmd = "echo '{0}' |".format(b)
        cmd += " intersectBed -a {0} -b stdin".format(self.gapfile)
        gaps = list(BedLine(x) for x in popen(cmd, debug=False))
        ngaps = len(gaps)

        gapsexpanded = []
        GeneDensity = 10000.  # assume 10Kb per gene
        for gap in gaps:
            gap_bp = int(gap.score)
            gap_ids = int(round(gap_bp / GeneDensity))
            gapsexpanded += [gap] * gap_ids

        lines = sorted(info + gapsexpanded, key=lambda x: x.start)

        message += "between bp: {0} - {1}, there are {2} gaps (total {3} ids)".\
                format(start_bp, end_bp, ngaps, len(lines))

        needed = lines
        stride = Stride(needed, available)
        conf = stride.conf
        message += " stride: {0}".format(conf)
        print >> sys.stderr, message

        nneeded = len(needed)
        if conf is None: # prefix rule - prepend version number for spills
            magic = 400000  # version 4
            firstdigit = 100000
            step = 10  # stride for the prefixed ids
            rank = start_id + magic
            if rank > magic + firstdigit:
                rank -= firstdigit
            available = []
            while len(available) != nneeded:
                rank += step
                if (current_chr, rank) in self.black:  # avoid blacklisted ids
                    continue
                available.append(rank)

        else: # follow the best stride
            available = stride.available
            if start_id == 0:  # follow right flank at start of chr
                available = available[- nneeded:]
            else:  # follow left flank otherwise
                available = available[:nneeded]

        # Finally assign the ids
        assert len(needed) == len(available)
        for b, rank in zip(needed, available):
            name = gene_name(current_chr, rank)
            print >> sys.stderr, "\t".join((str(b), name))
            id_table[b.accn] = name
            self.black.add((current_chr, rank))
        print >> sys.stderr


def main():

    actions = (
        ('rename', 'rename genes for annotation release'),
        # perform following actions on list files
        ('reindex', 'reindex isoforms per gene locus'),
        ('publocus', 'create pub_locus identifiers according to GenBank specs'),
        # Medicago gene renumbering
        ('annotate', 'annotation new bed file with features from old'),
        ('renumber', 'renumber genes for annotation updates'),
        ('instantiate', 'instantiate NEW genes tagged by renumber'),
        ('plot', 'plot gene identifiers along certain chromosome'),
        # External gene prediction programs
        ('augustus', 'convert augustus output into gff3'),
        ('tRNAscan', 'convert tRNAscan-SE output into gff3'),
            )
    p = ActionDispatcher(actions)
    p.dispatch(globals())


def plot(args):
    """
    %prog plot tagged.new.bed chr1

    Plot gene identifiers along a particular chromosome, often to illustrate the
    gene id assignment procedure.
    """
    from jcvi.graphics.base import plt, savefig
    from jcvi.graphics.chromosome import ChromosomeMap

    p = OptionParser(plot.__doc__)
    p.add_option("--firstn", type="int", help="Only plot the first N genes")
    p.add_option("--ymax", type="int", help="Y-axis max value")
    p.add_option("--log", action="store_true",
                help="Write plotting data [default: %default]")
    opts, args, iopts = p.set_image_options(args, figsize="6x4")

    if len(args) != 2:
        sys.exit(not p.print_help())

    taggedbed, chr = args
    bed = Bed(taggedbed)
    beds = list(bed.sub_bed(chr))
    old, new = [], []
    i = 0
    for b in beds:
        accn = b.extra[2]
        if "te" in accn:
            continue

        accn, tag = accn.split("|")
        if tag == "OVERLAP":
            continue

        c, r = atg_name(accn)
        if tag == "NEW":
            new.append((i, r))
        else:
            old.append((i, r))
        i += 1

    ngenes = i
    assert ngenes == len(new) + len(old)

    logging.debug("Imported {0} ranks on {1}.".format(ngenes, chr))
    fig = plt.figure(1, (iopts.w, iopts.h))
    root = fig.add_axes([0, 0, 1, 1])

    xstart, xend = .2, .8
    ystart, yend = .2, .8
    pad = .02

    ngenes = opts.firstn or ngenes
    ymax = opts.ymax or 500000

    title = "Assignment of Medtr identifiers"
    if opts.ymax:
        subtitle = "{0}, first {1} genes".format(chr, ngenes)
    else:
        subtitle = "{0}, {1} genes ({2} new)".format(chr, ngenes, len(new))

    chr_map = ChromosomeMap(fig, root, xstart, xend, ystart, yend, pad, 0,
                        ymax, 5, title, subtitle)

    ax = chr_map.axes

    if opts.log:
        from jcvi.utils.table import write_csv
        header = ["x", "y"]
        write_csv(header, new, filename=chr + ".new")
        write_csv(header, old, filename=chr + ".old")

    x, y = zip(*new)
    ax.plot(x, y, "b,")
    x, y = zip(*old)
    ax.plot(x, y, "r,")

    # Legends
    ymid = (ystart + yend) / 2
    y = ymid + pad
    root.plot([.2], [y], "r.", lw=2)
    root.text(.2 + pad, y, "Existing Medtr ids", va="center", size=10)
    y = ymid - pad
    root.plot([.2], [y], "b.", lw=2)
    root.text(.2 + pad, y, "Newly instantiated ids", va="center", size=10)

    ax.set_xlim(0, ngenes)
    ax.set_ylim(0, ymax)
    ax.set_axis_off()

    root.set_xlim(0, 1)
    root.set_ylim(0, 1)
    root.set_axis_off()

    image_name = chr + ".identifiers." + iopts.format
    savefig(image_name, dpi=iopts.dpi, iopts=iopts)


def instantiate(args):
    """
    %prog instantiate tagged.bed blacklist.ids big_gaps.bed

    instantiate NEW genes tagged by renumber.
    """
    p = OptionParser(instantiate.__doc__)
    opts, args = p.parse_args(args)

    if len(args) != 3:
        sys.exit(not p.print_help())

    taggedbed, blacklist, gapsbed = args
    r = NameRegister()
    r.get_blacklist(blacklist)
    r.get_gaps(gapsbed)

    # Run through the bed, identify stretch of NEW ids to instantiate,
    # identify the flanking FRAMEs, interpolate!
    bed = Bed(taggedbed)
    outputbed = taggedbed.rsplit(".", 1)[0] + ".new.bed"
    fw = open(outputbed, "w")

    tagkey = lambda x: x.rsplit("|", 1)[-1]
    for chr, sbed in bed.sub_beds():
        if "chr" not in chr:
            continue

        sbed = list(sbed)

        ranks = []
        for i, s in enumerate(sbed):
            nametag = s.extra[2]
            tag = tagkey(nametag)

            if tag in (NEW, FRAME):
                ranks.append((i, nametag))

        blocks = []
        for tag, names in groupby(ranks, key=lambda x: tagkey(x[-1])):
            names = list(names)
            if tag == NEW:
                blocks.append((tag, [sbed[x[0]] for x in names]))
            else:
                start, end = names[0][-1], names[-1][-1]
                start, end = atg_name(start, retval="rank"), atg_name(end, retval="rank")
                blocks.append((tag, [start, end]))

        id_table = {}  # old to new name conversion
        for i, (tag, info) in enumerate(blocks):
            if tag != NEW:
                continue

            start_id = 0 if i == 0 else blocks[i - 1][1][-1]
            end_id = start_id + 10000 if i == len(blocks) -1 \
                        else blocks[i + 1][1][0]

            r.allocate(info, chr, start_id, end_id, id_table)

        # Output new names
        for i, s in enumerate(sbed):
            nametag = s.extra[2]
            name, tag = nametag.split("|")

            if tag == NEW:
                assert name == '.'
                name = id_table[s.accn]
            elif tag == OVERLAP:
                if name in id_table:
                    name = id_table[name]

            s.extra[2] = "|".join((name, tag))
            print >> fw, s

    fw.close()


def atg_name(name, retval="chr,rank", trimpad0=True):
    atg_name_pat = re.compile(r"""
            ^(?P<locus>
                (?:(?P<prefix>\D+[\D\d\D])\.?)(?P<chr>[\d|C|M]+)?(?P<sep>[A-z]+)(?P<rank>\d+)
            )
            \.?(?P<iso>\d+)?
            """, re.VERBOSE)

    seps = ["g", "te", "trna", "s", "u"]
    pad0s = ["chr", "rank"]

    if name is not None:
        m = re.match(atg_name_pat, name)
        if m is not None and m.group('sep').lower() in seps:
            retvals = []
            for grp in retval.split(","):
                val = get_number(m.group(grp)) \
                        if trimpad0 and grp in pad0s \
                        else m.group(grp)
                retvals.append(val)

            return (x for x in retvals)
    else:
        return (None for x in retval.split(","))


def gene_name(current_chr, x, prefix="Medtr", sep="g", pad0=6):
    return "{0}{1}{2}{3:0{4}}".format(prefix, current_chr, sep, x, pad0)


def prepare(bedfile):
    """
    Remove prepended tags in gene names.
    """
    pf = bedfile.rsplit(".", 1)[0]
    abedfile = pf + ".a.bed"
    bbedfile = pf + ".b.bed"
    fwa = open(abedfile, "w")
    fwb = open(bbedfile, "w")

    bed = Bed(bedfile)
    seen = set()
    for b in bed:
        accns = b.accn.split(";")
        new_accns = []
        for accn in accns:
            if ":" in accn:
                method, a = accn.split(":", 1)
                if method in ("liftOver", "GMAP", ""):
                    accn = a
            if accn in seen:
                logging.error("Duplicate id {0} found. Ignored.".format(accn))
                continue

            new_accns.append(accn)
            b.accn = accn
            print >> fwa, b
            seen.add(accn)

        b.accn = ";".join(new_accns)
        print >> fwb, b
    fwa.close()
    fwb.close()


def renumber(args):
    """
    %prog renumber Mt35.consolidated.bed > tagged.bed

    Renumber genes for annotation updates.
    """
    from jcvi.algorithms.lis import longest_increasing_subsequence
    from jcvi.utils.grouper import Grouper

    p = OptionParser(renumber.__doc__)
    p.add_option("--pad0", default=6, type="int",
                 help="Pad gene identifiers with 0 [default: %default]")
    p.add_option("--prefix", default="Medtr",
                 help="Genome prefix [default: %default]")

    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(not p.print_help())

    bedfile, = args

    pf = bedfile.rsplit(".", 1)[0]
    abedfile = pf + ".a.bed"
    bbedfile = pf + ".b.bed"
    if need_update(bedfile, (abedfile, bbedfile)):
        prepare(bedfile)

    mbed = Bed(bbedfile)
    g = Grouper()
    for s in mbed:
        accn = s.accn
        g.join(*accn.split(";"))

    bed = Bed(abedfile)
    for chr, sbed in bed.sub_beds():
        if "chr" not in chr:
            continue

        current_chr = get_number(chr)
        ranks = []

        gg = set()
        for s in sbed:
            accn = s.accn
            achr, arank = atg_name(accn)
            if achr != current_chr:
                continue
            ranks.append(arank)
            gg.add(accn)

        lranks = longest_increasing_subsequence(ranks)
        print >> sys.stderr, current_chr, len(sbed), "==>", len(ranks), \
                    "==>", len(lranks)

        granks = set(gene_name(current_chr, x) for x in lranks) | \
                 set(gene_name(current_chr, x, sep="te") for x in lranks)

        tagstore = {}
        for s in sbed:
            achr, arank = atg_name(s.accn)
            accn = s.accn
            if accn in granks:
                tag = (accn, FRAME)
            elif accn in gg:
                tag = (accn, RETAIN)
            else:
                tag = (".", NEW)

            tagstore[accn] = tag

        # Find cases where genes overlap
        for s in sbed:
            accn = s.accn
            gaccn = g[accn]
            tags = [((tagstore[x][-1] if x in tagstore else NEW), x) for x in gaccn]
            group = [(PRIORITY.index(tag), x) for tag, x in tags]
            best = min(group)[-1]

            if accn != best:
                tag = (best, OVERLAP)
            else:
                tag = tagstore[accn]

            print "\t".join((str(s), "|".join(tag)))


def annotate(args):
    """
    %prog annotate new.bed old.bed 2> log

    Annotate the `new.bed` with features from `old.bed` for the purpose of
    gene numbering.

    Ambiguity in ID assignment can be resolved by either of the following 2 methods:
    - `alignment`: make use of global sequence alignment score (calculated by `needle`)
    - `overlap`: make use of overlap length (calculated by `intersectBed`)

    Transfer over as many identifiers as possible while following guidelines:
    http://www.arabidopsis.org/portals/nomenclature/guidelines.jsp#editing

    Note: Following RegExp pattern describes the structure of the identifier
    assigned to features in the `new.bed` file.

    new_id_pat = re.compile(r"^\d+\.[cemtx]+\S+")

    Examples: 23231.m312389, 23231.t004898, 23231.tRNA.144
    Adjust the value of `new_id_pat` manually as per your ID naming conventions.
    """
    from jcvi.utils.grouper import Grouper

    valid_resolve_choices = ["alignment", "overlap"]

    p = OptionParser(annotate.__doc__)
    p.add_option("--resolve", default="alignment", choices=valid_resolve_choices,
                 help="Resolve ID assignment based on a certain metric" \
                        + " [default: %default]")
    p.add_option("--atg_name", default=False, action="store_true",
                help="Specify is locus IDs in `new.bed` file follow ATG nomenclature" \
                        + " [default: %default]")

    g1 = OptionGroup(p, "Optional parameters (alignment):\n" \
            + "Use if resolving ambiguities based on sequence `alignment`")
    g1.add_option("--pid", dest="pid", default=35., type="float",
            help="Percent identity cutoff [default: %default]")
    g1.add_option("--score", dest="score", default=250., type="float",
            help="Alignment score cutoff [default: %default]")
    p.add_option_group(g1)

    g2 = OptionGroup(p, "Optional parameters (overlap):\n" \
            + "Use if resolving ambiguities based on `overlap` length\n" \
            + "Parameters equivalent to `intersectBed`")
    g2.add_option("-f", dest="f", default=0.5, type="float",
            help="Minimum overlap fraction (0.0 - 1.0) [default: %default]")
    g2.add_option("-r", dest="r", default=False, action="store_true",
            help="Require fraction overlap to be reciprocal [default: %default]")
    g2.add_option("-s", dest="s", default=True, action="store_true",
            help="Require same strandedness [default: %default]")
    p.add_option_group(g2)

    opts, args = p.parse_args(args)

    if len(args) != 2:
        sys.exit(not p.print_help())

    nbedfile, obedfile = args
    npf, opf = nbedfile.rsplit(".", 1)[0], obedfile.rsplit(".", 1)[0]

    # Make consolidated.bed
    cbedfile = "consolidated.bed"
    if not os.path.isfile(cbedfile):
        consolidate(nbedfile, obedfile, cbedfile)
    else:
        logging.warning("`{0}` already exists. Skipping step".format(cbedfile))

    logging.warning("Resolving ID assignment ambiguity based on `{0}`".\
            format(opts.resolve))

    if opts.resolve == "alignment":
        # Get pairs and prompt to run needle
        pairsfile = "nw.pairs"
        scoresfile = "nw.scores"
        if not os.path.isfile(pairsfile):
            get_pairs(cbedfile, pairsfile)
        else:
            logging.warning("`{0}` already exists. Checking for needle output".\
                    format(pairsfile))

        # If needle scores do not exist, prompt user to run needle
        if not os.path.isfile(scoresfile):
            logging.error("`{0}` does not exist. Please process {1} using `needle`".\
                    format(scoresfile, pairsfile))
            sys.exit()
    else:
        scoresfile = "ovl.scores"
        # Calculate overlap length using intersectBed
        calculate_ovl(nbedfile, obedfile, opts, scoresfile)

    logging.warning("`{0}' exists. Storing scores in memory".\
            format(scoresfile))
    scores = read_scores(scoresfile, opts)

    # Iterate through consolidated bed and
    # filter piles based on score
    abedline = {}

    cbed = Bed(cbedfile)
    g = Grouper()
    for c in cbed:
        accn = c.accn
        g.join(*accn.split(";"))

    nbedline = {}
    nbed = Bed(nbedfile)
    for line in nbed: nbedline[line.accn] = line

    splits = set()
    for chr, chrbed in nbed.sub_beds():
        abedline, splits = annotate_chr(chr, chrbed, g, scores, nbedline, abedline, opts, splits)

    if splits is not None:
        abedline = process_splits(splits, scores, nbedline, abedline)

    abedfile = npf + ".annotated.bed"
    afh = open(abedfile, "w")
    for accn in abedline:
        print >> afh, abedline[accn]
    afh.close()

    sort([abedfile, "-i"])


def calculate_ovl(nbedfile, obedfile, opts, scoresfile):
    from pybedtools import BedTool
    nbedtool = BedTool(nbedfile)
    obedtool = BedTool(obedfile)

    ab = nbedtool.intersect(obedtool, wao=True, f=opts.f, r=opts.r, s=opts.s)
    cmd = """cut -f4,5,10,13 | awk -F $'\t' 'BEGIN { OFS = FS } ($3 != "."){ print $1,$3,$2,$4; }'"""
    sh(cmd, infile=ab.fn, outfile=scoresfile)


def read_scores(scoresfile, opts):
    scores = {}
    fp = must_open(scoresfile)
    for row in fp:
        (new, old, identity, score) = row.strip().split("\t")
        old = re.sub('\.\d+$', '', old)
        if opts.resolve == "alignment":
            match = re.search("\d+\/\d+\s+\(\s*(\d+\.\d+)%\)", identity)
            pid = match.group(1)
            if float(pid) < opts.pid or float(score) < opts.score:
                continue
        else:
            pid = identity

        if new not in scores:
            scores[new] = []

        scores[new].append((new, old, pid, score))

    return scores


def annotate_chr(chr, chrbed, g, scores, nbedline, abedline, opts, splits):
    current_chr = get_number(chr)

    for line in chrbed:
        accn = line.accn
        if accn not in g or (opts.atg_name and "chr" not in chr):
            abedline[accn] = line
            continue

        gaccns = g[accn]
        new = [a for a in gaccns if re.search(new_id_pat, a)]
        newgrp = ";".join(sorted(new))

        if accn in scores:
            scores[accn] = sorted(scores[accn], key=lambda x: x[1])
            scores[accn] = sorted(scores[accn], key=lambda x: float(x[3]), reverse=True)

            accns = []
            print >> sys.stderr, accn
            for elem in scores[accn]:
                print >> sys.stderr, "\t" + ", ".join([str(x)\
                        for x in elem[1:]])
                if opts.atg_name:
                    achr, arank = atg_name(elem[1])
                    if not achr or achr != current_chr:
                        continue

                accns.append(elem[1])
                if len(new) > 1:
                    if newgrp not in scores: scores[newgrp] = []
                    scores[newgrp].append(elem)
                else:
                    accns[0:0] = [accn]
                    line.accn = ";".join([str(x) for x in accns])
                if len(scores[accn]) > 1: break

        if len(new) > 1:
            splits.add(newgrp)
        else:
            abedline[line.accn] = line

    return abedline, splits


def process_splits(splits, scores, nbedline, abedline):
    for newgrp in splits:
        new = newgrp.split(";")
        print >> sys.stderr, new
        if newgrp in scores:
            best = {}
            scores[newgrp] = sorted(scores[newgrp], key=lambda x: (x[0], x[1]))
            scores[newgrp] = sorted(scores[newgrp], key=lambda x: float(x[3]), reverse=True)

            for elem in scores[newgrp]:
                if elem[1] not in best:
                    best[elem[1]] = elem[0]

            for n in new:
                line = nbedline[n]
                if n in scores:
                    accns = set()
                    scores[n] = sorted(scores[n], key=lambda x: x[1])
                    scores[n] = sorted(scores[n], key=lambda x: float(x[3]), reverse=True)
                    accns.add(n)
                    print >> sys.stderr, "\t" + n
                    for elem in scores[n]:
                        if not elem[0] == n:
                            continue
                        print >> sys.stderr, "\t\t" + ", ".join([str(x)\
                                for x in elem[1:]])
                        if elem[1] in best and n == best[elem[1]]:
                            accns.add(elem[1])
                            accns = sorted(accns)
                            line.accn = ";".join([str(x) for x in accns])
                            break
                abedline[line.accn] = line
        else:
            for n in new:
                abedline[n] = nbedline[n]

    return abedline


def get_pairs(cbedfile, pairsfile):
    fp = open(pairsfile, "w")
    bed = Bed(cbedfile)
    for b in bed:
        if ";" in b.accn:
            genes = b.accn.split(";")
            new = [x for x in genes if re.search(new_id_pat, x)]
            old = [x for x in genes if not re.search(new_id_pat, x)]
            for a, b in product(new, old):
                print >> fp, "\t".join((a, b))

    fp.close()


def consolidate(nbedfile, obedfile, cbedfile):
    from pybedtools import BedTool
    nbedtool = BedTool(nbedfile)
    obedtool = BedTool(obedfile)

    ab = nbedtool.intersect(obedtool, s=True, u=True)
    ba = obedtool.intersect(nbedtool, s=True, u=True)

    cmd = "cat {0} {1} | sort -k1,1 -k2,2n".format(ab.fn, ba.fn)
    fp = popen(cmd)
    ovl = BedTool(fp.readlines())

    abmerge = ovl.merge(s=True, nms=True, scores="mean").sort()
    cmd = "cat {0}".format(abmerge.fn)
    fp = popen(cmd, debug=False)
    ovl = BedTool(fp.readlines())

    notovl = nbedtool.intersect(ovl.sort(), s=True, v=True)

    infile = "{0} {1}".format(notovl.fn, ovl.fn)
    tmpfile = "/tmp/reformat.{0}.bed".format(os.getpid())
    cmd = "sort -k1,1 -k2,2n"
    sh(cmd, infile=infile, outfile=tmpfile)

    fp = open(cbedfile, "w")
    bed = Bed(tmpfile)
    for b in bed:
        if ";" in b.accn:
            accns = set()
            for accn in b.accn.split(";"):
                accns.add(accn)
            b.accn = ";".join(accns)
        print >> fp, b
    fp.close()
    os.remove(tmpfile)

    sort([cbedfile, "-i"])


def rename(args):
    """
    %prog rename genes.bed [gaps.bed]

    Rename genes for annotation release.

    For genes on chromosomes (e.g. the 12th gene on C1):
    Bo1g00120

    For genes on scaffolds (e.g. the 12th gene on unplaced Scaffold00285):
    Bo00285s120

    The genes identifiers will increment by 10. So assuming no gap, these are
    the consecutive genes:
    Bo1g00120, Bo1g00130, Bo1g00140...
    Bo00285s120, Bo00285s130, Bo00285s140...

    When we encounter gaps, we would like the increment to be larger. For example,
    Bo1g00120, <gap>, Bo1g01120...

    Gaps bed file is optional.
    """
    import string

    p = OptionParser(rename.__doc__)
    p.add_option("-a", dest="gene_increment", default=10, type="int",
                 help="Increment for continuous genes [default: %default]")
    p.add_option("-b", dest="gap_increment", default=1000, type="int",
                 help="Increment for gaps [default: %default]")
    p.add_option("--pad0", default=6, type="int",
                 help="Pad gene identifiers with 0 [default: %default]")
    p.add_option("--spad0", default=4, type="int",
                 help="Pad gene identifiers on small scaffolds [default: %default]")
    p.add_option("--prefix", default="Bo",
                 help="Genome prefix [default: %default]")
    p.add_option("--jgi", default=False, action="store_true",
                 help="Create JGI style identifier PREFIX.NN[G|TE]NNNNN.1" + \
                      " [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) not in (1, 2):
        sys.exit(not p.print_help())

    genebed = args[0]
    gapbed = args[1] if len(args) == 2 else None
    prefix = opts.prefix
    gene_increment = opts.gene_increment
    gap_increment = opts.gap_increment

    genes = Bed(genebed)
    if gapbed:
        fp = open(gapbed)
        for row in fp:
            genes.append(BedLine(row))

    genes.sort(key=genes.key)
    idsfile = prefix + ".ids"
    newbedfile = prefix + ".bed"
    gap_increment -= gene_increment
    assert gap_increment >= 0

    if opts.jgi:
        prefix += "."
    fw = open(idsfile, "w")
    for chr, lines in groupby(genes, key=lambda x: x.seqid):
        lines = list(lines)
        pad0 = opts.pad0 if len(lines) > 1000 else opts.spad0
        isChr = chr[0].upper() == 'C'
        digits = "".join(x for x in chr if x in string.digits)
        gs = "g" if isChr else "s"
        pp = prefix + digits + gs
        idx = 0
        if isChr:
            idx += gap_increment

        for r in lines:
            isGap = r.strand not in ("+", "-")
            if isGap:
                idx += gap_increment
                continue
            else:
                idx += gene_increment
            accn = pp + "{0:0{1}d}".format(idx, pad0)
            oldaccn = r.accn
            print >> fw, "\t".join((oldaccn, accn))
            r.accn = accn

    genes.print_to_file(newbedfile)
    logging.debug("Converted IDs written to `{0}`.".format(idsfile))
    logging.debug("Converted bed written to `{0}`.".format(newbedfile))


def reindex(args):
    """
    %prog reindex idsfile > idsfiles.reindex

    Given a list of gene model identifier(following ATG naming conventions), reindex
    all the model IDs with new isoform indices

    Example output:
	Medtr2g100130.1         Medtr2g100130.1		SAME
	Medtr2g100130.3         Medtr2g100130.2		CHANGED
	Medtr2g100130.4         Medtr2g100130.3		CHANGED
	Medtr2g100130.5         Medtr2g100130.4		CHANGED
	Medtr2g100130.9         Medtr2g100130.5		CHANGED
	Medtr2g100130.11        Medtr2g100130.6		CHANGED
	Medtr2g100130.12        Medtr2g100130.7		CHANGED
	Medtr2g100130.13        Medtr2g100130.8		CHANGED

    Last column has 3-flags: "SAME" (Isoform index remains the same),
                             "CHANGED" (Isoform index has changed),
                         and "CHECK" (Isoform index for primary isoform is missing).
    """
    p = OptionParser(reindex.__doc__)
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(not p.print_help())

    index = {}
    idsfile, = args
    fp = must_open(idsfile)
    for row in fp:
        locus, iso = atg_name(row, retval="locus,iso")
        if None in (locus, iso):
            logging.warning("{0} is not a valid gene model identifier".format(row))
            continue
        if locus not in index:
            index[locus] = set()

        index[locus].add(int(iso))

    for locus in index:
        index[locus] = sorted(index[locus])
        l = len(index[locus])
        new = range(1,l+1)
        for idx, (i, ni) in enumerate(zip(index[locus], new)):
            flag = "SAME" if i == ni else "CHANGED"
            if idx == 0 and i != 1:
                flag = "CHECK"
            print "\t".join(x for x in ("{0}.{1}".format(locus, i), \
                                        "{0}.{1}".format(locus, ni), \
                                        flag))


def publocus(args):
    """
    %prog publocus idsfile > idsfiles.publocus

    Given a list of model identifiers, convert each into a GenBank approved
    pub_locus.

    Example output:
    Medtr1g007020.1		MTR_1g007020
    Medtr1g007030.1		MTR_1g007030
    Medtr1g007060.1		MTR_1g007060A
    Medtr1g007060.2		MTR_1g007060B
    """
    from jcvi.utils.cbook import AutoVivification

    p = OptionParser(publocus.__doc__)
    p.add_option("--locus_tag", default="MTR_",
                 help="GenBank locus tag [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(not p.print_help())

    locus_tag = opts.locus_tag

    index = AutoVivification()
    idsfile, = args
    fp = must_open(idsfile)
    for row in fp:
        locus, chrom, sep, rank, iso = atg_name(row, retval="locus,chr,sep,rank,iso")
        if None in (locus, chrom, sep, rank, iso):
            logging.warning("{0} is not a valid gene model identifier".format(row))
            continue
        if locus not in index.keys():
            pub_locus = gene_name(chrom, rank, prefix=locus_tag, sep=sep)
            index[locus]['pub_locus'] = pub_locus
            index[locus]['isos'] = set()

        index[locus]['isos'].add(int(iso))

    for locus in index:
        pub_locus = index[locus]['pub_locus']
        Index[locus]['isos'] = sorted(index[locus]['isos'])
        if len(index[locus]['isos']) > 1:
            new = [chr(n+64) for n in index[locus]['isos'] if n < 27]
            for i, ni in zip(index[locus]['isos'], new):
                print "\t".join(x for x in ("{0}.{1}".format(locus, i), \
                                            "{0}{1}".format(pub_locus, ni)))
        else:
            print "\t".join(x for x in ("{0}.{1}".format(locus, index[locus]['isos'][0]), \
                                        pub_locus))


def augustus(args):
    """
    %prog augustus augustus.gff3 > reformatted.gff3

    AUGUSTUS does generate a gff3 (--gff3=on) but need some refinement.
    """
    from jcvi.formats.gff import Gff

    p = OptionParser(augustus.__doc__)
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(not p.print_help())

    ingff3, = args
    gff = Gff(ingff3)
    for g in gff:
        if g.type not in ("gene", "transcript", "CDS"):
            continue

        if g.type == "transcript":
            g.type = "mRNA"

        print g


def tRNAscan(args):
    """
    %prog tRNAscan all.trna > all.trna.gff3

    Convert tRNAscan-SE output into gff3 format.

    Sequence                tRNA            Bounds          tRNA    Anti    Intron Bounds   Cove
    Name            tRNA #  Begin           End             Type    Codon   Begin   End     Score
    --------        ------  ----            ------          ----    -----   -----   ----    ------
    23231           1       335355          335440          Tyr     GTA     335392  335404  69.21
    23231           2       1076190         1076270         Leu     AAG     0       0       66.33

    Conversion based on PERL one-liner in:
    <https://github.com/sujaikumar/assemblage/blob/master/README-annotation.md>
    """
    from jcvi.formats.gff import sort

    p = OptionParser(tRNAscan.__doc__)
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(not p.print_help())

    trnaout, = args
    gffout = trnaout + ".gff3"
    fp = open(trnaout)
    fw = open(gffout, "w")

    fp.next()
    fp.next()
    row = fp.next()
    assert row.startswith("--------")

    for row in fp:
        atoms = [x.strip() for x in row.split("\t")]
        contig, trnanum, start, end, aa, codon, \
                intron_start, intron_end, score = atoms

        start, end = int(start), int(end)
        orientation = '+'
        if start > end:
            start, end = end, start
            orientation = '-'

        source = "tRNAscan"
        type = "tRNA"
        if codon == "???":
            codon = "XXX"

        comment = "ID={0}.tRNA.{1};Name=tRNA-{2} (anticodon: {3})".\
                format(contig, trnanum, aa, codon)

        print >> fw, "\t".join(str(x) for x in (contig, source, type, start,\
                            end, score, orientation, ".", comment))

    fw.close()
    sort([gffout, "-i"])


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
Convert delly output to BED format.
"""

import sys
import logging

from jcvi.formats.base import BaseFile, read_until, must_open
from jcvi.apps.base import OptionParser, ActionDispatcher


class DelLine (object):

    def __init__(self, line):
        args = line.strip().split("\t")
        self.seqid = args[0]
        self.start = int(args[1]) + 1
        self.end = int(args[2])
        self.size = int(args[3])
        assert self.size == self.end - self.start + 1
        self.supporting_pairs = int(args[4])
        self.avg_mapping_quality = float(args[5])
        self.accn = args[6]

    @property
    def bedline(self):
        return "\t".join(str(x) for x in (self.seqid,
                          self.start - 1, self.end, self.accn,
                          self.supporting_pairs, "+"))


class Delly (BaseFile):

    def __init__(self, filename):
        super(Delly, self).__init__(filename)

    def __iter__(self):
        fp = must_open(self.filename)
        while True:
            read_until(fp, "-----")
            nextline = fp.readline()
            nextline = fp.readline()
            if not nextline.strip():
                break
            d = DelLine(nextline)
            yield d

    def write_bed(self, bedfile="stdout"):
        fw = must_open(bedfile, "w")
        for d in self:
            print >> fw, d.bedline
        logging.debug("File written to `{0}`.".format(bedfile))


def main():

    actions = (
        ('bed', 'Convert del.txt to del.bed'),
            )
    p = ActionDispatcher(actions)
    p.dispatch(globals())


def bed(args):
    """
    %prog bed del.txt

    Convert `del.txt` to BED format. DELLY manual here:
    <http://www.embl.de/~rausch/delly.html>

    Deletion:
    chr, start, end, size, #supporting_pairs, avg._mapping_quality, deletion_id
    chr1, 10180, 10509, 329, 75, 15.8667, Deletion_Sample_00000000
    """
    p = OptionParser(bed.__doc__)
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(not p.print_help())

    delt, = args
    dt = Delly(delt)
    dt.write_bed("del.bed")


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
CLC bio assembly file CAS, and the tabular format generated by `assembly_table
-n -s -p`
"""

import os.path as op
import sys
import logging

from Bio import SeqIO

from jcvi.formats.base import LineFile
from jcvi.formats.sizes import Sizes
from jcvi.apps.base import OptionParser, ActionDispatcher, sh, need_update


class CasTabLine (LineFile):
    """
    The table generate by command `assembly_table -n -s -p`
    from clcbio assembly cell
    """
    def __init__(self, line):
        args = line.split()
        self.readnum = args[0]  # usually integer or `-`
        self.readname = args[1]
        self.readlen = int(args[-10])
        # 0-based indexing
        self.readstart = int(args[-9])
        if self.readstart >= 0:
            self.readstart += 1

        self.readstop = int(args[-8])
        self.refnum = int(args[-7])

        self.refstart = int(args[-6])
        if self.refstart >= 0:
            self.refstart += 1

        self.refstop = int(args[-5])

        self.is_reversed = (int(args[-4]) == 1)
        self.strand = '-' if self.is_reversed else '+'

        self.nummatches = int(args[-3])
        self.is_paired = (int(args[-2]) == 1)
        self.score = int(args[-1])

    @property
    def bedline(self):
        return "\t".join(str(x) for x in (self.refnum,
            self.refstart - 1, self.refstop, self.readname,
            self.score, self.strand))


def main():

    actions = (
        ('txt', "convert CAS file to tabular output using `assembly_table`"),
        ('split', 'split CAS file into smaller CAS using `sub_assembly`'),
        ('bed', 'convert cas tabular output to bed format'),
        ('pairs', 'print paired-end reads of cas tabular output'),
        ('info', 'print the number of read mapping using `assembly_info`'),
            )
    p = ActionDispatcher(actions)
    p.dispatch(globals())


def info(args):
    """
    %prog info casfile <fastafile>

    Wraps around `assembly_info` and get the following block.

    General info:
    Read info:
    Coverage info:

    In particular, the read info will be reorganized so that it shows the
    percentage of unmapped, mapped, unique and multi-hit reads.

    When --coverage is used, the program expects a second fastafile to replace
    the contig IDs with real ones.

    RPKM = 10^9 x C / NL, which is really just simply C/N

    C = the number of mappable reads that felt onto the gene's exons
    N = total number of mappable reads in the experiment
    L = the sum of the exons in base pairs.
    """
    from jcvi.utils.cbook import percentage

    p = OptionParser(info.__doc__)
    p.add_option("--coverage", default=False, action="store_true",
            help="Generate coverage output, replacing IDs [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) not in (1, 2):
        sys.exit(not p.print_help())

    casfile = args[0]
    pf = casfile.rsplit(".", 1)[0]

    if opts.coverage:
        assert len(args) == 2, "You need a fastafile when using --coverage"
        coveragefile = pf + ".coverage"
        fw = open(coveragefile, "w")

    infofile = pf + ".info"
    cmd = "assembly_info {0}".format(casfile)
    if not op.exists(infofile):
        sh(cmd, outfile=infofile)

    inreadblock = False
    incontigblock = False

    fp = open(infofile)
    row = fp.readline()
    while row:
        if row.startswith("Read info:"):
            inreadblock = True
        elif row.startswith("Contig info:"):
            incontigblock = True

        # Following looks like a hack, but to keep compatible between
        # CLC 3.20 and CLC 4.0 beta
        if inreadblock:
            atoms = row.split('s')

            last = atoms[-1].split()[0] if len(atoms) > 1 else "0"
            srow = row.strip()

            if srow.startswith("Reads"):
                reads = int(last)
            if srow.startswith("Unmapped") or srow.startswith("Unassembled"):
                unmapped = int(last)
            if srow.startswith("Mapped") or srow.startswith("Assembled"):
                mapped = int(last)
            if srow.startswith("Multi"):
                multihits = int(last)

            if row.startswith("Coverage info:"):
                # Print the Read info: block
                print "Read info:"
                assert mapped + unmapped == reads

                unique = mapped - multihits
                print
                print "Total reads: {0}".format(reads)
                print "Unmapped reads: {0}".format(percentage(unmapped, reads, False))
                print "Mapped reads: {0}".format(percentage(mapped, reads, False))
                print "Unique reads: {0}".format(percentage(unique, reads, False))
                print "Multi hit reads: {0}".\
                        format(percentage(multihits, reads, False))
                print
                inreadblock = False

        if incontigblock and opts.coverage:

            fastafile = args[1]
            s = Sizes(fastafile)
            while row:
                atoms = row.split()
                if len(atoms) == 4 and atoms[0][0] != "C":  # Contig
                    # Contig       Sites       Reads     Coverage
                    contig, sites, reads, coverage = atoms
                    contig = int(contig) - 1
                    size = s.sizes[contig]
                    contig = s.ctgs[contig]
                    assert size == int(sites)

                    # See formula above
                    rpkm = 1e9 * int(reads) / (size * mapped)
                    print >> fw, "\t".join((contig, sites, reads,
                        "{0:.1f}".format(rpkm)))

                row = fp.readline()

        row = fp.readline()


def txt(args):
    """
    %prog txt casfile

    convert binary CAS file to tabular output using CLC assembly_table
    """
    p = OptionParser(txt.__doc__)
    p.add_option("-m", dest="multi", default=False, action="store_true",
        help="report multi-matches [default: %default]")
    opts, args = p.parse_args(args)

    if len(args) != 1:
        sys.exit(p.print_help())

    casfile, = args
    txtfile = casfile.replace(".cas", ".txt")
    assert op.exists(casfile)

    cmd = "assembly_table -n -s -p "
    if opts.multi:
        cmd += "-m "
    cmd += casfile
    sh(cmd, outfile=txtfile)

    return txtfile


def split(args):
    """
    %prog split casfile 1 10

    split the binary casfile by using CLCbio `sub_assembly` program, the two
    numbers are starting and ending index for the `reference`; useful to split
    one big assembly per contig
    """
    p = OptionParser(split.__doc__)
    opts, args = p.parse_args(args)

    if len(args) != 3:
        sys.exit(p.print_help())

    casfile, start, end = args
    start = int(start)
    end = int(end)

    split_cmd = "sub_assembly -a {casfile} -o sa.{i}.cas -s {i} " + \
        "-e sa.{i}.pairs.fasta -f sa.{i}.fragments.fasta -g sa.{i}.ref.fasta"

    for i in range(start, end + 1):
        cmd = split_cmd.format(casfile=casfile, i=i)
        sh(cmd)


def check_txt(casfile):
    """
    Check to see if the casfile is already converted to txtfile with txt().
    """
    if casfile.endswith(".cas"):
        castabfile = casfile.replace(".cas", ".txt")
        if need_update(casfile, castabfile):
            castabfile = txt([casfile])
        else:
            logging.debug("File `{0}` found.".format(castabfile))
    else:
        castabfile = casfile

    return castabfile


def bed(args):
    """
    %prog bed casfile <fastafile>

    Convert the CAS or CASTAB format into bed format. If fastafile given, the
    sequential IDs in the casfile will be replaced by FASTA header.
    """
    p = OptionParser(bed.__doc__)
    opts, args = p.parse_args(args)

    nargs = len(args)
    if nargs not in (1, 2):
        sys.exit(not p.print_help())

    hasfastafile = (nargs == 2)
    casfile = args[0]
    castabfile = check_txt(casfile)

    if hasfastafile:
        fastafile = args[1]
        refnames = [rec.id for rec in SeqIO.parse(fastafile, "fasta")]

    fp = open(castabfile)
    bedfile = castabfile.rsplit(".", 1)[0] + ".bed"

    if need_update(castabfile, bedfile):
        fw = open(bedfile, "w")
        for row in fp:
            b = CasTabLine(row)
            if b.readstart == -1:
                continue
            if hasfastafile:
                b.refnum = refnames[b.refnum]
            print >> fw, b.bedline

        logging.debug("File written to `{0}`.".format(bedfile))
    else:
        logging.debug("File `{0}` up to date. Computation skipped.".\
                      format(bedfile))

    return bedfile


def pairs(args):
    """
    See __doc__ for OptionParser.set_pairs().
    """
    import jcvi.formats.bed

    p = OptionParser(pairs.__doc__)
    p.set_pairs()
    opts, targs = p.parse_args(args)

    if len(targs) != 1:
        sys.exit(not p.print_help())

    casfile, = targs
    bedfile = bed([casfile])
    args[args.index(casfile)] = bedfile

    return jcvi.formats.bed.pairs(args)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
%prog fpcfile

Parser for FPC physical map file, writes a tabular file to stdout
"""

import sys
import logging

from jcvi.apps.base import OptionParser
from jcvi.formats.base import read_until, read_block


bac_tag = "BAC :"


class FpcRecord (object):

    def __init__(self):
        self.bac_name = ''
        self.ctg_name = ''
        self.map_left = 0
        self.map_right = 0
        self.cosmid = ''
        self.cosmid_match = ''
        self.gel_number = ''
        self.fp_number = ''
        self.bands = ''
        self.probes = []
        self.cre_date = ''
        self.mod_date = ''
        self.remark = []

    def __str__(self):
        return '\t'.join((self.bac_name, self.ctg_name,
            str(self.map_left), str(self.map_right),
            self.bands, ','.join(self.probes), ','.join(self.remark)))


class FpcReader (object):
    """
    parse fpc file, usage:

    >>> reader = FpcReader()
    >>> for rec in reader:
            print rec
    """
    def __init__(self, fpcfile):
        self._handle = open(fpcfile)

    def __iter__(self):
        line = self._handle.readline()
        if not line.startswith(bac_tag):
            read_until(self._handle, bac_tag)

        for header, seq in read_block(self._handle, bac_tag):

            rec = FpcRecord()
            assert header.startswith(bac_tag)
            rec.bac_name = header.split('\"')[1]

            for line in seq:
                if line.startswith("Map"):
                    rec.ctg_name = line.split('\"')[1]
                    if "Left" in line:
                        rec.map_left = line.split("Left")[1].split()[0]
                        rec.map_left = int(float(rec.map_left))
                    if "Right" in line:
                        rec.map_right = line.split("Right")[1].split()[0]
                        rec.map_right = int(float(rec.map_right))
                if line.startswith("Gel_number"):
                    rec.gel_number = line.split()[-1]
                if line.startswith("Fp_number"):
                    rec.fp_number = line.split()[-1]
                if line.startswith("Bands"):
                    rec.bands = line.split()[-1]
                if line.startswith("match_to_cosmid"):
                    rec.cosmid = line.split('\"')[1]
                    rec.cosmid_match = line.split("_match_")[0]
                if line.startswith("Positive"):
                    rec.probes.append(line.split('\"')[1])
                if line.startswith("Fpc_remark"):
                    rec.remark.append(line.split('\"')[1])
                if line.startswith("Creation_date"):
                    rec.cre_date = line.split('_date')[1].strip()
                if line.startswith("Modified_date"):
                    rec.mod_date = line.split('_date')[1].strip()

            yield rec


def main(fpcfile):

    fw = sys.stdout
    f = FpcReader(fpcfile)

    # first several lines are comments
    header = '\t'.join(('bac_name', 'ctg_name', 'map_left', 'map_right',
        'bands', 'probes', 'remark'))
    print >> fw, header

    recs = list(f)
    logging.debug("%d records parsed" % len(recs))

    recs.sort(key=lambda x: (x.ctg_name, x.map_left, x.map_right))
    for rec in recs:
        print >> fw, rec


if __name__ == '__main__':

    p = OptionParser(__doc__)
    opts, args = p.parse_args()

    if len(args) != 1:
        sys.exit(p.print_help())

    fpcfile, = args

    main(fpcfile)
